Package: mvnma
Title: Multivariate Network Meta-Analysis using Bayesian Methods
Version: 0.1-0
Date: 2026-05-15
Depends: R (>= 4.0.0), meta (>= 8.3-0), netmeta (>= 3.4-0)
Imports: R2jags, coda, dplyr, magrittr, matrixStats, rlist, ggplot2,
        forcats
Authors@R: c(person("Theodoros", "Evrenoglou",
                    role = c("aut", "cre"),
                    email = "theodoros.evrenoglou@uniklinik-freiburg.de",
                    comment = c(ORCID = "0000-0003-3336-8058")),
             person("Guido", "Schwarzer",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-6214-9087")))
Maintainer: Theodoros Evrenoglou <theodoros.evrenoglou@uniklinik-freiburg.de>
URL: https://github.com/TEvrenoglou/mvnma
Description: Tools to conduct Bayesian multivariate network meta-analysis providing
 - the single correlation coefficient model by Efthimiou et al. (2015) <doi:10.1093/biostatistics/kxu030>;
 - per-outcome treatment hierarchies using the surface under the cumulative ranking curve (SUCRA), the probability of best value, or median (or mean) ranks (Salanti et al., 2011) <doi:10.1016/j.jclinepi.2010.03.016>;
 - across-outcomes benefit-risk assessment using the VišeKriterijumska Optimizacija I Kompromisno Rešenje (VIKOR) method (Opricovic & Tzeng, 2004) <doi:10.1016/S0377-2217(03)00020-1>;
 - convergence checks using trace plots, density plots, or the R-hat statistic;
 - forest plots of treatment estimates, scatter plots of per-outcome rankings, Hasse diagrams (Carlsen & Bruggemann, 2014) <doi:10.1002/cem.2569> to visualize the partial order of the treatments across all outcomes.
License: GPL (>= 2)
Encoding: UTF-8
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-15 12:43:41 UTC; evrenogl
Author: Theodoros Evrenoglou [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3336-8058>),
  Guido Schwarzer [aut] (ORCID: <https://orcid.org/0000-0001-6214-9087>)
Repository: CRAN
Date/Publication: 2026-05-19 09:50:02 UTC
Built: R 4.6.0; ; 2026-05-19 11:18:37 UTC; unix

Package: mvnma
Title: Multivariate Network Meta-Analysis using Bayesian Methods
Version: 0.2-0
Date: 2026-07-09
Depends: R (>= 4.0.0), meta (>= 8.3-0), netmeta (>= 3.4-0)
Imports: R2jags, coda, dplyr, magrittr, matrixStats, rlist, ggplot2,
        forcats
Authors@R: c(person("Theodoros", "Evrenoglou",
                    role = c("aut", "cre"),
                    email = "theodoros.evrenoglou@uniklinik-freiburg.de",
                    comment = c(ORCID = "0000-0003-3336-8058")),
             person("Guido", "Schwarzer",
                    role = "aut",
                    comment = c(ORCID = "0000-0001-6214-9087")))
Maintainer: Theodoros Evrenoglou <theodoros.evrenoglou@uniklinik-freiburg.de>
URL: https://github.com/TEvrenoglou/mvnma
Description: Tools to conduct Bayesian multivariate network meta-analysis providing
 - the single correlation coefficient model by Efthimiou et al. (2015) <doi:10.1093/biostatistics/kxu030>;
 - per-outcome treatment hierarchies using the surface under the cumulative ranking curve (SUCRA), the probability of best value, or median (or mean) ranks (Salanti et al., 2011) <doi:10.1016/j.jclinepi.2010.03.016>;
 - across-outcomes benefit-risk assessment using the VišeKriterijumska Optimizacija I Kompromisno Rešenje (VIKOR) method (Opricovic & Tzeng, 2004) <doi:10.1016/S0377-2217(03)00020-1>;
 - convergence checks using trace plots, density plots, or the R-hat statistic;
 - forest plots of treatment estimates, scatter plots of per-outcome rankings, Hasse diagrams (Carlsen & Bruggemann, 2014) <doi:10.1002/cem.2569> to visualize the partial order of the treatments across all outcomes.
License: GPL (>= 2)
Encoding: UTF-8
NeedsCompilation: no
Config/roxygen2/version: 8.0.0
Packaged: 2026-07-09 12:22:21 UTC; evrenogl
Author: Theodoros Evrenoglou [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3336-8058>),
  Guido Schwarzer [aut] (ORCID: <https://orcid.org/0000-0001-6214-9087>)
Repository: CRAN
Date/Publication: 2026-07-09 16:40:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(as.mcmc,mvnma)
S3method(forest,mvnma)
S3method(hasse,mvrank)
S3method(heatplot,mvrank)
S3method(plot,mvrank)
S3method(print,mvnma)
S3method(print,mvrank)
S3method(print,spiechart)
S3method(print,vikor)
S3method(spiechart,mvrank)
S3method(vikor,matrix)
S3method(vikor,mvrank)
export(as.mcmc)
export(linechart)
export(mvnma)
export(mvrank)
export(spiechart)
export(vikor)
importFrom(R2jags,jags)
importFrom(coda,as.mcmc)
importFrom(coda,as.mcmc.list)
importFrom(dplyr,"%>%")
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n_distinct)
importFrom(dplyr,pull)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(forcats,fct_rev)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_colourbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(graphics,text)
importFrom(magrittr,"%<>%")
importFrom(matrixStats,colSds)
importFrom(meta,forest)
importFrom(meta,gs)
importFrom(meta,metagen)
importFrom(meta,pairwise)
importFrom(netmeta,hasse)
importFrom(netmeta,heatplot)
importFrom(netmeta,netposet)
importFrom(netmeta,rankogram)
importFrom(rlist,list.cbind)
importFrom(rlist,list.rbind)
importFrom(stats,complete.cases)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(utils,combn)
importFrom(utils,packageVersion)

#' Scatter plot to visualize the ranking of two outcomes in a multivariate network meta-analysis
#' 
#' @description
#' Draw a scatter plot in the active graphics window.
#' 
#' @param x An object of class \code{\link{mvrank}}.
#' @param which A mandatory numeric vector of length 2 specifying which
#'   outcomes should be plotted. For example, setting "outcome = c(2, 3)"
#'   implies that a scatter plot will be generated plotting the rankings
#'   of outcomes 2 and 3.
#' @param pos Position of treatment labels.
#' @param cex.point a numeric value specifying the size of the points
#'   (default: 1)
#' @param cex.label a numeric value specifying the size of the point labels in
#'   the plot (default: 0.7)
#' @param pch a vector of plotting characters or symbols (default: 19)
#' @param xlim the x limits of the plot
#' @param ylim the y limits of the plot
#' @param \dots Additional arguments for \code{\link{plot}} function.
#'
#' @return
#' A scatter plot is shown in the active graphics window.
#' 
#' @keywords hplot
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Rank treatments using SUCRAs (default)
#' ranks12 <- mvrank(mvnma12, small.values = c("undes", "undes"))
#' ranks12
#' 
#' # Visualize SUCRAs in a scatter plot with outcome 1 on the x-axis and
#' # outcome 2 on the y-axis
#' plot(ranks12)
#' 
#' # Visualize SUCRAs in a scatter plot with outcome 2 on the x-axis and
#' # outcome 1 on the y-axis
#' plot(ranks12, which = 2:1)
#' 
#' @method plot mvrank
#' @export

plot.mvrank <- function(x, which = 1:2, pos = 1,
                        cex.point = 1, cex.label = 0.7, pch = 19,
                        xlim = c(0, 1), ylim = c(0, 1),
                        ...) {
  
  chkclass(x, "mvrank")
  #
  n.outcome <- length(names(x))
  common_trts <- attr(x, "common_trts")
  #
  chknumeric(which, min = 1, max = n.outcome, length = 2)
  chknumeric(cex.point, min = 0, zero = TRUE)
  chknumeric(cex.label, min = 0, zero = TRUE)
  chknumeric(pch, min = 1, zero = TRUE)
  chknumeric(xlim, length = 2)
  chknumeric(ylim, length = 2)
  #
  first <- which[1]
  second <- which[2]
  
  # Get rid of warning "no visible binding for global variable"
  treat <- NULL
  
  outcomes <- names(x)[c(first, second)]
  #
  dat1 <- x[[first]]
  names(dat1)[1:2] <- c("treat", "rank1")
  dat1$out1 <- outcomes[1]
  #
  dat1 %<>% filter(treat %in% common_trts)
  
  dat2 <- x[[second]]
  names(dat2)[1:2] <- c("treat", "rank2")
  dat2$out2 <- outcomes[2]
  #
  dat2 %<>% filter(treat %in% common_trts)
  #
  dat <- merge(dat1, dat2, by = "treat", all.x = TRUE, all.y = TRUE)
  #
  plot(dat$rank1, dat$rank2, main = "",
       cex = cex.point,
       xlab = outcomes[1], ylab = outcomes[2],
       pch = pch, xlim = xlim, ylim = ylim, ...)
  #
  text(dat$rank1, dat$rank2, labels = dat$treat,
       cex = cex.label, pos = pos, col = "black")
  #
  invisible(NULL)
}

mvdata <- function(p) {
  
  if (is.list.pairwise(p) != "pairwise")
    stop("Argument 'p' must be a list of 'pairwise' objects.")
  
  data_format <- create_data(p)
  jags_data <- make_jags_data(data_format)
  #
  class(jags_data) <- "mvdata"
  attr(jags_data, "structured_data") <- data_format
  
  sm <- vector("character")
  #
  for (i in seq_along(p))
    sm[i] <- attr(p[[i]], "sm") 
  #
  attr(jags_data, "sm") <- sm
  
  jags_data
}

#' mvnma: Brief overview of methods
#'
#' @description
#' R package \bold{mvnma} provides R functions for Bayesian multivariate
#' network meta-analysis (mvNMA). The mvNMA model supported by this package
#' refers to the single correlation coefficient model, interpreted as an
#' amalgam of within- and across-outcome correlations (Efthimiou et al., 2015)
#' which is a generalisation of Riley et al. (2008).
#' In this way, the model does not depend on the extraction of within-study
#' outcome correlations, which are seldom reported at the study level.
#' 
#' DuMouchel priors assuming constant relative treatment effects across
#' outcomes and enabling information sharing can be used (DuMouchel & Harris,
#' 1983). This may improve precision but can introduce bias when outcomes from
#' different domains (e.g., efficacy and safety) are analyzed jointly.
#' 
#' The treatment effect estimates
#' and confidence intervals can be summarised both in terms of per-outcome
#' treatment hierarchies and in terms of an across-outcomes benefit-risk
#' assessment. The former is possible using ranking methods such as the surface
#' under the cumulative ranking curve (SUCRA) (Salanti et al., 2011), the
#' probability of best value, and median (or mean) ranks, each accompanied by a
#' credible interval.
#' 
#' A benefit-risk assessment is possible through the VišeKriterijumska
#' Optimizacija I Kompromisno Rešenje (VIKOR) method (Opricovic & Tzeng, 2004;
#' Opricovic, 2011). This approach, originally proposed in the field of
#' multi-criteria decision analysis, uses a deterministic algorithm to provide
#' an amalgamated treatment hierarchy across outcomes and explicitly identify
#' the set of treatments that offer the best compromise between benefits and
#' harms across all outcomes.
#' 
#' Since the output of the method relies on Markov Chain Monte Carlo (MCMC)
#' sampling, convergence can be checked using a series of diagnostics, including
#' trace plots, density plots, and the R-hat statistic. Finally, this package
#' offers the option to visualise the results of the mvNMA model through forest
#' plots, which display the treatment effect estimates; scatter plots, which
#' show the per-outcome rankings for any pair of outcomes; and Hasse diagrams
#' (Carlsen & Bruggemann, 2014), which visualise the partial order of treatments
#' across all outcomes (Rücker & Schwarzer, 2017), as well as alternative
#' methods to yield a between-outcomes hierarchy, such as the spie chart method
#' (Daly et al., 2020).
#' 
#' @details
#' The R package \bold{mvnma} provides the following functions:
#' \itemize{
#' \item Function \code{\link{mvnma}} to perform a Bayesian multivariate
#'   network meta-analysis.
#' \item Function \code{\link{mvrank}} to get outcome-specific treatment
#'  rankings.
#' \item Function \code{\link{vikor}} to rank treatments across all outcomes
#'   using the VIKOR multi-criteria decision analysis method. Additionally,
#'   the function evaluates the concrete conditions defined by the VIKOR method
#'   and identifies the set of treatments that offer the best compromise
#'   between benefits and harms across all outcomes.
#' \item Function \code{\link{forest.mvnma}} to visualize the results of the
#'   mvNMA model in terms of treatment effect estimates.
#' \item Function \code{\link{plot.mvrank}} to visualize per outcome ranking
#'   results for any pair of outcomes.
#' \item Function \code{\link{hasse.mvrank}} to visualize the partial order of
#'   the treatment across all outcomes.
#' \item Function \code{\link{heatplot.mvrank}} to visualize in a heatplot
#'   the results in terms of outcome specific rankings.
#' \item Function \code{\link{linechart}} to visualize the results of the three
#'   metrics calculated by the VIKOR method.
#' \item Function \code{\link{as.mcmc.mvnma}} an auxiliary function to extract
#'   an MCMC object. This makes any \bold{mvnnma} object compatible with the
#'   convergence checks performed by the R package \bold{coda}.
#'  \item Function \code{\link{spiechart}} to calculate a between outcomes
#'    hierarchy using the spie charts method.
#' }
#' 
#' Type \code{help(package = "mvnma")} for a listing of R functions
#' available in \bold{mvnma}.
#'
#' Type \code{citation("mvnma")} on how to cite \bold{mvnma}
#' in publications.
#'
#' To report problems and bugs, please send an email to Theodoros
#' Evrenoglou <theodoros.evrenoglou@uniklinik-freiburg.de>.
#'
#' The development version of \bold{mvnma} is available on GitHub
#' \url{https://github.com/TEvrenoglou/mvnma}.
#'
#' @name mvnma-package
#'
#' @author Theodoros Evrenoglou <theodoros.evrenoglou@@uniklinik-freiburg.de>,
#'   Guido Schwarzer <guido.schwarzer@@uniklinik-freiburg.de>
#' 
#' @references
#' Carlsen L, Bruggemann R (2014):
#' Partial order methodology: a valuable tool in chemometrics.
#' \emph{Journal of Chemometrics},
#' \bold{28}, 226--34
#' 
#' Daly CH, Mbuagbaw L, Thabane L, Straus SE, Hamid JS (2020):
#' Spie charts for quantifying treatment effectiveness and safety in multiple
#' outcome network meta-analysis: a proof-of-concept study.
#' \emph{BMC Med Res Methodol}, \bold{20}, 266
#' 
#' DuMouchel WH, Harris JE (1983):
#' Bayes methods for combining the results of cancer studies in humans and
#' other species.
#' \emph{Journal of the American Statistical Association},
#' \bold{78}, 293--308
#' 
#' Efthimiou O, Mavridis D, Riley RD, Cipriani A, Salanti G (2015):
#' Joint synthesis of multiple correlated outcomes in networks of interventions.
#' \emph{Biostatistics}, 
#' \bold{16}, 84--97
#'
#' Opricovic S, Tzeng GH (2004):
#' Compromise solution by MCDM methods: A comparative analysis of VIKOR and
#' TOPSIS.
#' \emph{European Journal of Operational Research},
#' \bold{156}, 445--55
#' 
#' Opricovic S (2011):
#' Fuzzy VIKOR with an application to water resources planning. 
#' \emph{Expert Systems with Applications}, 
#' \bold{38}, 12983--90
#' 
#' Riley RD, Thompson JR, Abrams KR (2008):
#' An alternative model for bivariate random-effects meta-analysis when the
#' within-study correlations are unknown.
#' \emph{Biostatistics},
#' \bold{9}, 172--86
#' 
#' Rücker G, Schwarzer G (2017):
#' Resolve conflicting rankings of outcomes in network meta-analysis:
#' Partial ordering of treatments.
#' \emph{Research Synthesis Methods},
#' \bold{8}, 526--36
#'                                             
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' @keywords package
#'
#' @importFrom R2jags jags
#' @importFrom coda as.mcmc as.mcmc.list
#' @importFrom meta forest gs metagen pairwise
#' @importFrom netmeta hasse netposet rankogram heatplot
#' @importFrom matrixStats colSds
#' @importFrom dplyr %>% all_of any_of arrange bind_rows bind_cols desc distinct filter group_by mutate rename select pull n_distinct
#' @importFrom magrittr %<>%
#' @importFrom rlist list.cbind list.rbind
#' @importFrom graphics text
#' @importFrom stats complete.cases quantile relevel
#' @importFrom utils combn packageVersion
#' @importFrom ggplot2 ggplot aes geom_tile geom_line geom_point geom_text scale_fill_gradient guides guide_colourbar guide_legend labs xlab ylab ylim scale_y_discrete theme theme_void theme_minimal element_text element_blank
#' @importFrom forcats fct_rev
#' @export as.mcmc

"_PACKAGE"

NULL

spiechart_internal <- function(x, weights = NULL){
  
  # Get rid of warning 'Undefined global functions or variables'
  #
  area <- NULL
  
  if (is.null(weights))
    weights <- rep(1 /ncol(x), ncol(x))
  else if ((!is.null(weights) & length(weights) != ncol(x)))
    stop("Please provide weights for all outcomes.",
         call. = FALSE)
  #
  if (!is.null(weights) && sum(weights) != 1) {
    weights <- round(weights / sum(weights), digits = 2)
    #
    warning("Weights should always sum up to 1. To do so the given weights ",
            "are now standardized and the new weights are: ",
            paste(weights, collapse = ", "))
  }
  
  # Transform weights to sum up to 2 * pi
  #
  weights <- 2 * pi * weights
  
  n.trts <- nrow(x)
  res <- data.frame(area = rep(NA, n.trts))
  row.names(res) <- row.names(x)
  for (i in seq_len(n.trts))
    res$area[i] <- (1 / (2 * pi)) * sum(weights * x[i, ]^2)
  #
  res %<>% arrange(desc(area))
  #
  attr(res, "transformed.weights") <- weights
  #
  res
}

#' Auxiliary function to extract the Markov Chain Monte Carlo object
#' 
#' @description
#' Extract the Markov Chain Monte Carlo object from an R object of class
#' \code{mvnma}.
#' 
#' @param x An object of class \code{mvnma}.
#' @param drop.reference.group A logical indicating whether to drop the MCMC
#'   samples for the reference group.
#' @param \dots Additional arguments passed on to
#'   \code{\link[coda]{as.mcmc.list}}.
#' 
#' @return
#' The function returns the Markov Chain Monte Carlo object which is a list
#' of 'mcmc' objects.
#' 
#' @aliases as.mcmc.mvnma as.mcmc
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{mvnma}}
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' mcmc12 <- as.mcmc(mvnma12)
#' 
#' library(coda)
#' oldpar <- par(mfrow = c(3, 3))
#' # Traceplot
#' traceplot(mcmc12)
#' # Density plot
#' densplot(mcmc12)
#' # Traceplot and density plot together
#' plot(mcmc12)
#' 
#' # Do not print the trace plot for the reference group, i.e., placebo
#' mcmc12.drop <- as.mcmc(mvnma12, drop.reference.group = TRUE)
#' par(mfrow = c(2, 4))
#' # traceplot
#' traceplot(mcmc12.drop)
#' # density plot
#' densplot(mcmc12.drop)
#' # traceplot and density plot together
#' plot(mcmc12.drop)
#' 
#' par(oldpar)
#' 
#' @method as.mcmc mvnma
#' @export

as.mcmc.mvnma <- function(x, drop.reference.group = FALSE, ...) {
  chkclass(x, "mvnma")
  #
  chklogical(drop.reference.group)
  #
  trts <- attr(x, "trts")
  reference.group <- attr(x, "reference.group")
  #
  res <- as.mcmc.list(attr(x, "fit")$BUGSoutput, ...)
  #
  for (i in seq_along(res)) {
    nam <- colnames(res[[i]])
    #
    for (j in seq_along(trts)) {
      patt <- paste0("\\b(d\\d+)\\[", j, "\\]")
      repl <-  paste0("\\1[", trts[j], "\\]")
      #
      nam <- gsub(pattern = patt, replacement = repl, x = nam)
    }
    #
    colnames(res[[i]]) <- nam
  }
  #
  if (drop.reference.group) {
    for (i in seq_along(res)) {
      nam <- colnames(res[[i]])
      #
      patt <- paste0("\\b(d\\d+)\\[", reference.group, "\\]")
      #
      keep <- !grepl(pattern = patt, x = nam)
      #
      res[[i]] <- res[[i]][, keep, drop = FALSE]
    }
  }
  #
  res
}

#' Perform a Bayesian multivariate network meta-analysis using a
#' single-correlation coefficient model
#' 
#' @description
#' This function fits a Bayesian multivariate network meta-analysis model.
#' Currently, the function can simultaneously pool up to five outcomes.
#' Additionally, the studies to be included should be of maximum three arms.
#' 
#' @param \dots Either two to five pairwise objects or a single list with
#'   two to five pairwise objects.
#' @param reference.group A common reference treatment across all outcomes.
#' @param outclab An optional argument with labels for each outcome. If NULL,
#'   the each outcome is labelled as 'outcome_1', 'outcome_2' etc.
#' @param n.chains Number of Markov chains (default=4). 
#' @param n.domain Integer indicating the position of the last outcome in the 
#' first outcome domain (based on the order of the supplied pairwise objects). 
#' Used with `method = "DM"` to restrict information sharing within outcome 
#' domains. Ignored when `method = "standard"`. Default is `NULL`.
#' @param n.thin Thinning rate. Default is equal to
#'   \code{max(1, floor((n.iter - n.burnin) / 1000))}.
#' @param n.iter Number of iterations (default: 10000).
#' @param n.burnin Number of iterations for burn-in (default: 2000).
#' @param level The level used to calculate confidence intervals
#'   for network estimates.
#' @param scale.psi Values for the scale parameter(s) of the Half-Normal prior
#'   used for the heterogeneity parameters within each outcome. If NULL, all
#'   values are set to 1. If specified, it should have a length equal to the
#'   number of outcomes.
#' @param lower.rho Lower bounds for the Uniform prior(s) used for the
#'   correlation coefficient. If NULL all bounds are set to -1.
#' @param upper.rho Upper bounds for the Uniform prior(s) used for the
#'   correlation coefficient. If NULL all bounds are set to 1.
#' @param method A character string specifying the method to be used for model
#'   fitting. This can be either "standard" (default), referring to the
#'   standard bivariate model, or "DM", referring to the bivariate model based
#'   on the DuMouchel method. The argument can be abbreviated.
#' @param varTE.missing Assumed (very large) variance for outcomes not reported
#'   in a study. By default, the largest variance times 1000000 is used. This is
#'   the same value used for argument \code{seTE.ignore} in
#'   \code{\link[netmeta]{netimpact}} to mimicking the removal of individual
#'   studies from the network meta-analysis.
#' @param quiet A logical indicating whether to print information on the
#'   progress of the JAGS model fitting.
#' @param x An object of class \code{\link{mvnma}}.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.sd Minimal number of significant digits for standard
#'   deviations
#' @param print.sd A logical specifying whether standard deviations should be
#'   printed.
#' @param \dots Additional arguments (ignored)
#' 
#' @details
#' The multivariate network meta-analysis (mvNMA) model supported by this
#' package refers to the single correlation coefficient model, interpreted as
#' an amalgam of within- and across-outcome correlations
#' (Efthimiou et al., 2015) which is a generalisation of Riley et al. (2008).
#' 
#' The function \code{\link{mvnma}} expects two to five outcomes /
#' \code{\link[meta]{pairwise}} objects. A common reference treatment across
#' all outcomes is required to only show comparisons with the reference in
#' forest plots.
#' 
#' The Bayesian multivariate network meta-analysis model fitted in the
#' \bold{mvnma} package assumes uniform priors for the between-outcome
#' correlation coefficients. The lower and upper bounds of these priors can be
#' defined using the arguments `lower.rho` and `upper.rho`. If not set, the
#' model will assume a `Unif (-1, 1)` prior for all correlation coefficients.
#' For two outcomes, a single value can be provided for `lower.rho` and
#' `upper.rho`. For example, `lower.rho` = 0.5 and `upper.rho` = 1 for
#' rho12 ~ Unif (0.5, 1)).
#' For more than two outcomes, the order in which the bounds are provided
#' matters. For example, when pooling four outcomes, the lower and
#' upper bounds correspond to the following order of correlation coefficients:
#' (rho12, rho13, rho14, rho23, rho24, rho34).
#' 
#' Two types of priors for the treatment effect parameters are supported via 
#' the argument `method`. Setting `method = "standard"` fits an mvNMA model 
#' using non-informative normal priors (e.g., `N(0, 10^3)`).
#' 
#' Alternatively, `method = "DM"` specifies the DuMouchel prior, which assumes 
#' constant relative treatment effects across outcomes and enables information 
#' sharing (DuMouchel & Harris, 1983). This may improve precision but can
#' introduce bias when outcomes from different domains (e.g., efficacy and
#' safety) are analyzed jointly.
#' 
#' The argument `n.domain` can be used to restrict information sharing to 
#' predefined outcome domains. It indicates the position (based on the order 
#' of the supplied pairwise objects) of the last outcome in the first domain. 
#' For example, with four outcomes, setting `n.domain = 2` assigns the first 
#' two outcomes to one domain and the remaining outcomes to a second domain. 
#' In this case, information is shared only within domains.
#' 
#' By default, `n.domain = NULL`, in which case information is shared across 
#' all outcomes when `method = "DM"`. This may be appropriate when all outcomes 
#' belong to the same domain or when cross-domain sharing is justified.
#' 
#' The argument `n.domain` is ignored when `method = "standard"`.
#'  
#' @return
#' The function returns an 'mvnma' object. This consists of the results for each
#' outcome and the correlation coefficient estimates between the combined
#' outcomes. The outcome-specific estimates are expressed in the format of a
#' list (one for each outcome) which contains:
#' \itemize{
#' \item The basic estimates (i.e. treatment vs. reference.group) for each
#'   outcome.
#' \item The heterogeneity estimates for each outcome 
#' \item The posterior samples corresponding to the basic estimates.
#' }
#' 
#' @seealso \code{\link[meta]{pairwise}}
#' 
#' @references
#' DuMouchel WH, Harris JE (1983):
#' Bayes methods for combining the results of cancer studies in humans and
#' other species.
#' \emph{Journal of the American Statistical Association},
#' \bold{78}, 293--308
#' 
#' Efthimiou O, Mavridis D, Riley RD, Cipriani A, Salanti G (2015):
#' Joint synthesis of multiple correlated outcomes in networks of interventions.
#' \emph{Biostatistics}, 
#' \bold{16}, 84--97
#' 
#' Riley RD, Thompson JR, Abrams KR (2008):
#' An alternative model for bivariate random-effects meta-analysis when the
#' within-study correlations are unknown.
#' \emph{Biostatistics},
#' \bold{9}, 172--86
#' 
#' @examples
#' # Use 'pairwise' to obtain contrast based data for the first two outcomes
#' 
#' # Early response
#' pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' 
#' # Early remissions
#' pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' 
#' # Define outcome labels
#' outcomes <- c("Early_Response", "Early_Remission",
#'   "Adverse_events", "Loss_to_follow_up", "Loss_to_follow_up_AE")
#' 
#' # Fit the model combining only the two efficacy outcomes
#' # (note, we are using only 10 iterations and 2 burnins to reduce the
#' #  runtime of the example; in real applications use larger numbers)
#' set.seed(1910)
#' mvnma(pw1, pw2,
#'   reference.group = "Placebo", outclab = outcomes[1:2],
#'   n.iter = 10, n.burnin = 2)
#' 
#' \donttest{
#' # Use 'pairwise' to obtain contrast based data for the third to fifth
#' # outcome
#' 
#' # Adverse events
#' pw3 <- pairwise(treat = list(treatment1, treatment2,treatment3),
#'   event = list(ae1, ae2, ae3),  n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' 
#' # Loss to follow-up
#' pw4 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(loss1, loss2, loss3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' 
#' # Loss_to_follow_up_(AE)
#' pw5 <- pairwise(treat = list(treatment1, treatment2, treatment3),
#'   event = list(loss.ae1, loss.ae2, loss.ae3), n = list(n1, n2, n3),
#'   studlab = id, data = Linde2015, sm = "OR")
#' 
#' # Fit the model combining only the two efficacy outcomes
#' # (note, we are using only 100 iterations and 20 burnins to reduce the
#' #  runtime of the example; in real applications use larger numbers)
#' set.seed(1909)
#' mvnma12 <- mvnma(pw1, pw2,
#'   reference.group = "Placebo", outclab = outcomes[1:2],
#'   n.iter = 100, n.burnin = 20)
#' mvnma12
#' 
#' # Extract treatment effect estimates and heterogeneity for Early_Response 
#' mvnma12$Early_Response$basic_estimates
#' mvnma12$Early_Response$heterogeneity
#' 
#' # Extract outcome correlation
#' mvnma12$cor
#' 
#' # Plot the results for efficacy outcomes
#' forest(mvnma12)
#' 
#' # Print odds ratios for efficacy outcomes
#' outc <- names(mvnma12)[names(mvnma12) != "cor"]
#' #
#' for (i in outc) {
#'   cat(paste0("\nOutcome: ", i, "\n\n"))
#'   print(round(exp(mvnma12[[i]]$TE.random), 2))
#' }
#' 
#' # Fit the model combining all five outcomes
#' # (note, we are using only 100 iterations and 20 burnins to reduce the
#' #  runtime of the example; in real applications use larger numbers)
#' set.seed(1904)
#' mvnma_all <- mvnma(pw1, pw2, pw3, pw4, pw5,
#'   reference.group = "Placebo", outclab = outcomes,
#'   n.iter = 100, n.burnin = 20)
#' 
#' # Extract treatment effect estimates and heterogeneity for Early_Response 
#' mvnma_all$Early_Response$basic_estimates
#' mvnma_all$Early_Response$heterogeneity      
#' 
#' # Extract outcome correlation 
#' mvnma_all$cor
#' 
#' # Plot the results for all outcomes
#' forest(mvnma_all)
#' 
#' # Print odds ratios for all outcomes
#' outc <- names(mvnma_all)[names(mvnma_all) != "cor"]
#' #
#' for (i in outc) {
#'   cat(paste0("\nOutcome: ", i, "\n\n"))
#'   print(round(exp(mvnma_all[[i]]$TE.random), 2))
#' }
#' }
#' 
#' @export mvnma

mvnma <- function(...,
                  reference.group = NULL, outclab = NULL,   
                  n.domain = NULL,
                  n.chains = 4, n.iter = 10000, 
                  n.burnin = 2000, 
                  n.thin = max(1, floor((n.iter - n.burnin) / 1000)), 
                  level = gs("level.ma"),
                  scale.psi,
                  lower.rho, upper.rho,
                  method = "standard",
                  varTE.missing = NULL,
                  quiet = FALSE) {
  
  # Get rid of warning "no visible binding for global variable"
  studlab <- NULL
  #
  args <- list(...)
  #
  n.out <- length(args)
  #
  chknumeric(n.domain, min = 1, max = n.out)
  n.dom <- n.domain
  n.i <- seq_len(n.out)
  #
  if (n.out == 1) {
    if (inherits(args[[1]], "pairwise"))
      stop("Provide between two and five pairwise objects.",
           call. = FALSE)
    #
    if (!is.list(args[[1]]))
      stop("All elements of argument '...' must be of classes ",
           "'netmeta', 'netcomb', or 'discomb'.",
           call. = FALSE)
    #
    if (!inherits(args[[1]], "pairwise")) {
      n.out <- length(args[[1]])
      n.i <- seq_len(n.out)
      #
      args2 <- list()
      for (i in n.i)
        args2[[i]] <- args[[1]][[i]]
    }
    args <- args2
  }
  #  
  for (i in n.i) {
    if (!inherits(args[[i]], "pairwise"))
      stop("All elements of argument '...' must be of class ",
           "'pairwise'.",
           call. = FALSE)
  }
  #
  if (n.out < 2 | n.out > 5)
    stop("Provide between two and five pairwise objects.",
         call. = FALSE)
  #
  data <- mvdata(args)
  
  trts.list <- data$trts.list
  #
  chknull(reference.group)
  chklevel(level)
  chklogical(quiet)
  #
  method <- setchar(method, c("standard", "DM"))
  #
  # Extract number of outcomes
  #
  n.out <- ncol(data$var %>% select(-studlab))
  n.cor <- choose(n.out, 2)
  #
  miss.lower <- missing(lower.rho)
  miss.upper <- missing(upper.rho)
  miss.scale.psi <- missing(scale.psi)
  #
  if (!miss.lower)
    chknumeric(lower.rho, min = -1, max = 1, length = n.cor, NA.ok = FALSE)
  #
  if (!miss.upper)
    chknumeric(upper.rho, min = -1, max = 1, length = n.cor, NA.ok = FALSE)
  #
  if (!miss.lower & !miss.upper) {
    if (any(lower.rho >= upper.rho))
      stop("Values for argument 'lower.rho' must be smaller than values for ",
           "argument 'upper.rho'.",
           call. = FALSE)
  }
  
  #
  if (!miss.scale.psi)
    chknumeric(scale.psi,zero = TRUE, min = 0, length = n.out, NA.ok = FALSE)
  
  # Create bounds for correlation prior
  #
  if (miss.lower)
    lower.rho1 <- -1
  else
    lower.rho1 <- lower.rho[1]
  #
  if (miss.upper)
    upper.rho1 <- 1
  else
    upper.rho1 <- upper.rho[1]
  #
  if (n.out >= 3) {
    if (miss.lower) {
      lower.rho2 <- -1
      lower.rho3 <- -1
    }
    else {
      lower.rho2 <- lower.rho[2]
      lower.rho3 <- lower.rho[3]
    }
    #
    if (miss.upper) {
      upper.rho2 <- 1
      upper.rho3 <- 1
    }
    else {
      upper.rho2 <- upper.rho[2]
      upper.rho3 <- upper.rho[3]
    }
  }
  #
  if (n.out >= 4) {
    if (miss.lower) {
      lower.rho4 <- -1
      lower.rho5 <- -1
      lower.rho6 <- -1
    }
    else {
      lower.rho4 <- lower.rho[4]
      lower.rho5 <- lower.rho[5]
      lower.rho6 <- lower.rho[6]
    }
    #
    if (miss.upper) {
      upper.rho4 <- 1
      upper.rho5 <- 1
      upper.rho6 <- 1
    }
    else {
      upper.rho4 <- upper.rho[4]
      upper.rho5 <- upper.rho[5]
      upper.rho6 <- upper.rho[6]
    }
  }
  #
  if (n.out >= 5) {
    if (miss.lower) {
      lower.rho7  <- -1
      lower.rho8  <- -1
      lower.rho9  <- -1
      lower.rho10 <- -1
    }
    else {
      lower.rho7  <- lower.rho[7]
      lower.rho8  <- lower.rho[8]
      lower.rho9  <- lower.rho[9]
      lower.rho10 <- lower.rho[10]
    }
    #
    if (miss.upper) {
      upper.rho7  <- 1
      upper.rho8  <- 1
      upper.rho9  <- 1
      upper.rho10 <- 1
    }
    else {
      upper.rho7  <- upper.rho[7]
      upper.rho8  <- upper.rho[8]
      upper.rho9  <- upper.rho[9]
      upper.rho10 <- upper.rho[10]
    }
  }
  
  # Create values for the scale and precision of parameter psi
  
  if (miss.scale.psi) {
    scale.psi1 <- 1
    scale.psi2 <- 1
  }  
  else {
    scale.psi1 <- scale.psi[1]
    scale.psi2 <- scale.psi[2]
  }
  
  prec.psi1 <- 1 / scale.psi1^2
  
  prec.psi2 <- 1 / scale.psi2^2
  
  if (n.out >= 3) {
    if (miss.scale.psi)
      scale.psi3 <- 1
    else
      scale.psi3 <- scale.psi[3]
    #
    prec.psi3 <- 1 / scale.psi3^2
  }
  
  if (n.out >= 4) {
    if (miss.scale.psi)
      scale.psi4 <- 1
    else
      scale.psi4 <- scale.psi[4]
    #
    prec.psi4 <- 1 / scale.psi4^2
  }
  
  if (n.out >= 5) {
    if (miss.scale.psi)
      scale.psi5 <- 1
    else
      scale.psi5 <- scale.psi[5]
    #
    prec.psi5 <- 1 / scale.psi5^2
  }
  
  # Create outcome labels if not provided
  #
  if (is.null(outclab))
    outclab <- paste("outcome", seq_len(n.out), sep = "_")  
  else if (length(outclab) != n.out)
    stop("Please provide labels for all outcomes.")
  
  trts <- data$trts
  #
  ref <- unname(which(trts == reference.group))  
  
  multiarm <- ncol(data$T) > 2
  #
  dat_var <- data$var %>% filter(!duplicated(studlab))
  rownames(dat_var) <- dat_var$studlab
  dat_var %<>% select(-studlab)
  #
  control_matrix <- 1L * !is.na(dat_var)
  #
  if (is.null(varTE.missing)) {
    varTE.missing <-
      1000^2 * max(data$var %>% select(-studlab), na.rm = TRUE)
  }
  else {
    chknumeric(varTE.missing, min = 0, zero = TRUE, length = 1)
    #
    if (varTE.missing < max(data$var %>% select(-studlab), na.rm = TRUE))
      stop("The value provided for argument 'varTE.missing' must be larger ",
           "than the largest available variance in the dataset.",
           .call = FALSE)
  }
  #
  data$var[is.na(data$var)] <- varTE.missing
  
  run.data <- list(
    y = data$y,
    #
    var1 = data$var$var1,
    var2 = data$var$var2,
    var3 = NA,
    var4 = NA,
    var5 = NA,
    #
    control = control_matrix,
    #
    ref = ref,
    #
    k = data$k,
    k2 = data$k2,
    n = data$n,
    #
    treat1 = data$T[, 1], treat2 = data$T[, 2], treat3 = NA,
    #
    prec.psi1 = prec.psi1, prec.psi2 = prec.psi2,
    prec.psi3 = NA, prec.psi4 = NA, prec.psi5 = NA,
    #
    lower.rho1 = lower.rho1, upper.rho1 = upper.rho1,
    lower.rho2 = NA, upper.rho2 = NA,
    lower.rho3 = NA, upper.rho3 = NA,
    lower.rho4 = NA, upper.rho4 = NA,
    lower.rho5 = NA, upper.rho5 = NA,
    lower.rho6 = NA, upper.rho6 = NA,
    lower.rho7 = NA, upper.rho7 = NA,
    lower.rho8 = NA, upper.rho8 = NA,
    lower.rho9 = NA, upper.rho9 = NA,
    lower.rho10 = NA, upper.rho10 = NA)
  #
  if (n.out >= 3) {
    run.data$var3 <- data$var$var3
    #
    run.data$prec.psi3 <- prec.psi3
    #
    run.data$lower.rho2 <- lower.rho2
    run.data$lower.rho3 <- lower.rho3
    #
    run.data$upper.rho2 <- upper.rho2
    run.data$upper.rho3 <- upper.rho3
  }
  #
  if (n.out >= 4) {
    run.data$var4 <- data$var$var4
    #
    run.data$prec.psi4 <- prec.psi4
    #
    run.data$lower.rho4 <- lower.rho4
    run.data$lower.rho5 <- lower.rho5
    run.data$lower.rho6 <- lower.rho6
    #
    run.data$upper.rho4 <- upper.rho4
    run.data$upper.rho5 <- upper.rho5
    run.data$upper.rho6 <- upper.rho6
  }
  #
  if (n.out >= 5) {
    run.data$var5 <- data$var$var5
    #
    run.data$prec.psi5 <- prec.psi5
    #
    run.data$lower.rho7 <- lower.rho7
    run.data$lower.rho8 <- lower.rho8
    run.data$lower.rho9 <- lower.rho9
    run.data$lower.rho10 <- lower.rho10
    #
    run.data$upper.rho7 <- upper.rho7
    run.data$upper.rho8 <- upper.rho8
    run.data$upper.rho9 <- upper.rho9
    run.data$upper.rho10 <- upper.rho10
  }
  #
  if (multiarm)
    run.data$treat3 <- data$T[, 3]
  else
    run.data$treat3 <- NULL
  #
  if (n.out == 2) {
    run.data$var3 <- run.data$var4 <- run.data$var5 <- NULL
    #
    run.data$prec.psi3 <- run.data$prec.psi4 <- run.data$prec.psi5 <- NULL
    #
    run.data$lower.rho2 <- run.data$lower.rho3 <- run.data$lower.rho4 <-
      run.data$lower.rho5 <- run.data$lower.rho6 <- run.data$lower.rho7 <-
      run.data$lower.rho8 <- run.data$lower.rho9 <- run.data$lower.rho10 <-
      NULL
    #
    run.data$upper.rho2 <- run.data$upper.rho3 <- run.data$upper.rho4 <-
      run.data$upper.rho5 <- run.data$upper.rho6 <- run.data$upper.rho7 <-
      run.data$upper.rho8 <- run.data$upper.rho9 <- run.data$upper.rho10 <-
      NULL
    #
    params <- c("d1", "d2", 
                "psi1", "psi2",
                "rho1")
    #
    model.code <- mvnma_code(n.out, method, multiarm, n.dom)
  }
  #
  else if (n.out == 3) {
    run.data$var4 <- run.data$var5 <- NULL
    #
    run.data$prec.psi4 <- run.data$prec.psi5 <- NULL
    #
    run.data$lower.rho4 <- run.data$lower.rho5 <- run.data$lower.rho6 <-
      run.data$lower.rho7 <- run.data$lower.rho8 <- run.data$lower.rho9 <-
      run.data$lower.rho10 <- NULL
    #
    run.data$upper.rho4 <- run.data$upper.rho5 <- run.data$upper.rho6 <-
      run.data$upper.rho7 <- run.data$upper.rho8 <- run.data$upper.rho9 <-
      run.data$upper.rho10 <- NULL
    #
    params <- c("d1", "d2", "d3", 
                "psi1", "psi2", "psi3",
                "rho1", "rho2", "rho3")
    #
    model.code <- mvnma_code(n.out, method, multiarm, n.dom)
  }
  #
  else if (n.out == 4) {
    run.data$var5 <- NULL
    #
    run.data$prec.psi5 <- NULL
    #
    run.data$lower.rho7 <- run.data$lower.rho8 <- run.data$lower.rho9 <-
      run.data$lower.rho10 <- NULL
    #
    run.data$upper.rho7 <- run.data$upper.rho8 <- run.data$upper.rho9 <-
      run.data$upper.rho10 <- NULL
    #
    params <- c("d1", "d2", "d3", "d4", 
                "psi1", "psi2", "psi3", "psi4",
                "rho1", "rho2", "rho3", "rho4", "rho5", "rho6")
    #
    model.code <- mvnma_code(n.out, method, multiarm, n.dom)
  }
  #
  else if (n.out == 5) {
    params <- c("d1", "d2", "d3", "d4", "d5",
                "psi1", "psi2", "psi3", "psi4", "psi5",
                "rho1", "rho2", "rho3", "rho4", "rho5", "rho6",
                "rho7", "rho8", "rho9", "rho10")
    #
    model.code <- mvnma_code(n.out, method, multiarm, n.dom)
  }
  #
  if (method == "DM") {
    if (is.null(n.domain)) {
      params <- c(params, "sigma")  
    }
    else{
      params <- c(params, "sigma1", "sigma2") 
    }
  }
  #
  if (!multiarm)
    run.data$k <- NULL
  
  
  #
  # Run Bayesian analysis
  #
  
  text_conn <- textConnection(model.code)
  on.exit(close(text_conn), add = TRUE)
  #
  fit <- jags(
    data = run.data,
    inits = NULL,
    #
    parameters.to.save = params,
    #
    n.chains = n.chains, n.iter = n.iter, 
    n.burnin = n.burnin, n.thin = n.thin,
    #
    DIC = FALSE,
    #
    model.file = text_conn,
    quiet = quiet)
  #
  samples <- fit$BUGSoutput$sims.list
  colnames(samples$d1) <- trts
  colnames(samples$d2) <- trts
  #
  # Manipulate the results and create suitable datasets
  #
  res <- gather_results(fit,
                        outcomes = outclab,
                        trts = trts,
                        trts.list = trts.list,
                        reference.group = reference.group,
                        level = level,
                        n.domain = n.domain,
                        method = method)
  #
  attr(res, "outcomes") <- outclab
  attr(res, "trts") <- trts
  attr(res,"n.domain") <- n.domain
  attr(res, "reference.group") <- reference.group
  attr(res, "level") <- level
  attr(res, "sm") <- attr(data, "sm")
  attr(res, "method.model") <- method
  attr(res, "model.code") <- model.code
  attr(res, "fit") <- fit
  attr(res, "params") <- params
  attr(res, "varTE.missing") <- varTE.missing
  #
  class(res) <- "mvnma"
  #
  res
}


#' @rdname mvnma
#' @method print mvnma
#' @export

print.mvnma <- function(x,
                        digits = gs("digits"),
                        digits.sd = gs("digits.sd"),
                        print.sd = FALSE,
                        ...) {
  
  chkclass(x, "mvnma")
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.sd, min = 0, length = 1)
  chklogical(print.sd)
  #
  level <- attr(x, "level")
  reference.group <- attr(x, "reference.group")
  method <- attr(x, "method")
  n.domain <- attr(x,"n.domain")
  #
  ci.lab <- paste0(round(100 * level, 1), "%-CI")
  #
  x <- x[names(x) != "cor"]
  #
  if (method == "DM") {
    if (is.null(n.domain)) {
      x <- x[names(x) != "sigma"]
    }
    else {
      x <- x[!(names(x) %in% c("sigma1","sigma2"))]
    }
  }
  nam <- names(x)
  
  # Get rid of warning "no visible binding for global variable"
  lower <- upper <- psi <- NULL
  #
  for (i in seq_along(nam)) {
    cat(paste0(if (i > 1) "\n" else "", "Outcome: ", nam[i], "\n\n"))
    #
    dat.i <- x[[i]]$basic_estimates
    dat.i <- dat.i[rownames(dat.i) != reference.group, ]
    #
    dat.i$mean <- formatN(dat.i$mean, digits = digits)
    #
    if (!print.sd)
      dat.i$sd <- NULL
    else
      dat.i$sd <- formatN(dat.i$sd, digits = digits.sd)
    #
    dat.i$lower <- formatCI(formatN(dat.i$lower, digits = digits),
                            formatN(dat.i$upper, digits = digits))
    dat.i %<>% select(-upper)
    names(dat.i)[names(dat.i) == "lower"] <- ci.lab
    #
    dat.i$Rhat <- formatN(dat.i$Rhat, digits = 4)
    #
    rownames(dat.i) <- paste0("d[", rownames(dat.i), "]")
    #
    prmatrix(dat.i, quote = FALSE, right = TRUE)
  }
  #
  invisible(NULL)
}

#' Hasse diagram
#' 
#' @description
#' This function generates a Hasse diagram for a partial order of treatment
#' ranks in a multivariate network meta-analysis.
#' 
#' @param x An object of class \code{\link{mvrank}}.
#' @param \dots Additional arguments passed on to
#'   \code{\link[netmeta]{hasse.netposet}}.
#' 
#' @details
#' Generate a Hasse diagram (Carlsen & Bruggemann, 2014) for a partial order of
#' treatment ranks in a network meta-analysis (Rücker & Schwarzer, 2017).
#' 
#' This R function is a wrapper function for
#' \code{\link[netmeta]{hasse.netposet}}.
#'
#' @return
#' A Hasse diagram is shown in the active graphics window.
#' 
#' @references
#' Carlsen L, Bruggemann R (2014):
#' Partial order methodology: a valuable tool in chemometrics.
#' \emph{Journal of Chemometrics},
#' \bold{28}, 226--34
#'
#' Rücker G, Schwarzer G (2017):
#' Resolve conflicting rankings of outcomes in network meta-analysis:
#' Partial ordering of treatments.
#' \emph{Research Synthesis Methods},
#' \bold{8}, 526--36
#'                                             
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Print odds ratios for efficacy outcomes
#' outc <- names(mvnma12)[names(mvnma12) != "cor"]
#' #
#' for (i in outc) {
#'   cat(paste0("\nOutcome: ", i, "\n\n"))
#'   print(round(exp(mvnma12[[i]]$TE.random), 2))
#' }
#' 
#' # Rank treatments using SUCRAs
#' ranks12 <- mvrank(mvnma12, method = "sucra",
#'   small.values = c("undes", "undes"))
#' ranks12
#' 
#' # Get the Hasse diagram for the efficacy outcomes
#' hasse(ranks12)
#' 
#' @method hasse mvrank
#' @export

hasse.mvrank <- function(x, ...) {
  
  if (!(attr(x, "method") %in% c("SUCRA", "pBV")))
    stop("Hasse diagram can only be produced for ",
         "'method = \"SUCRA\"' and 'method = \"pBV\"'.",
         call. = FALSE)
  
  # Get rid of warning "no visible binding for global variable"
  treatment <- NULL
  
  treats <- E <- new_treats <- vector("list")
  #
  for (i in seq_along(x))
    treats[[i]] <- x[[i]]$treatment  
  #
  all_treats <- unique(unlist(treats))
  
  for (i in seq_along(x)) {
    E[[i]] <- which(!(all_treats %in% x[[i]]$treatment))
    #
    if (length(E[[i]]) > 0) {
      new_treats[[i]] <- cbind.data.frame(all_treats[E[[i]]], NA)
      names(new_treats[[i]]) <- names(x[[i]])
      #
      x[[i]] <- rbind.data.frame(x[[i]], new_treats[[i]])
    }
    #
    x[[i]] %<>% arrange(treatment)
    row.names(x[[i]]) <- x[[i]]$treatment
    #
    x[[i]]$treatment <- NULL
  } 
  #
  ranking_mat <- list.cbind(x) 
  names(ranking_mat) <- names(x)
  #
  po <- netposet(ranking_mat, outcomes = names(ranking_mat))
  #
  hasse(po, ...)
  #
  invisible(NULL)
}

mvnma_code <- function(n.out, method, multiarm, n.dom) {
  
  chknumeric(n.out, min = 2, length = 1)
  method <- setchar(method, c("standard", "DM"))
  chklogical(multiarm)
  #
  txt <-
    paste0("model {\n",
           "  # k = total number of studies\n",
           "  # k2 = number of two-arm studies\n",
           "  # n = number of treatments\n\n")
  #
  # Files with variances, covariances and estimates
  #
  txt <- paste0(txt, code_covar_ests(n.out, multiarm))
  #
  txt <- paste0(txt, code_means(n.out, multiarm))
  #
  txt <- paste0(txt, "\n")
  #
  txt <- paste0(txt, code_priors(n.out, method,n.dom))
  #
  txt <- paste0(txt, "}\n")
  #
  txt
}

code_covar_ests <- function(n.out, multiarm) {
  
  txt <-
    paste0(
      "  #\n",
      "  #\n",
      "  # (1) Variances, covariances and estimates\n",
      "  #\n",
      "  #\n  \n")
  #
  txt <-
    paste0(txt,
           "  #\n",
           "  # Two-arm studies\n",
           "  #\n")
  #
  txt <- paste0(txt, "  for (i in 1:k2) {\n")
  #
  txt <-
    paste0(txt,
           "    #\n",
           "    # Variances\n",
           "    #\n")
  #
  for (i in seq_len(n.out)) {
    txt <-
      paste0(txt,
             "    S2[i, ", i, ", ", i, "] <- var", i, "[i] + ",
             "psi", i, ".sq\n")
  }
  #
  txt <-
    paste0(txt,
           "    #\n",
           "    # Covariances\n",
           "    #\n")
  #
  r <- 0
  #
  for (i in seq_len(n.out - 1)) {
    for (j in (i + 1):n.out) {
      r <- r + 1
      #
      txt <-
        paste0(txt,
               "    S2[i, ", i, ", ", j, "] <- ",
               "sqrt(S2[i, ", i, ", ", i, "]) * ",
               "sqrt(S2[i, ", j, ", ", j, "]) * ",
               "control", "[i, ", i, "] * ",
               "control", "[i, ", j, "] * ",
               "rho", r, "\n")
    }
  }
  #
  txt <- paste0(txt, "    #\n")
  #
  for (i in seq_len(n.out - 1)) {
    for (j in (i + 1):n.out) {
      txt <-
        paste0(txt,
               "    S2[i, ", j, ", ", i, "] <- S2[i, ", i, ", ", j, "]\n")
    }
  }
  #
  txt <-
    paste0(txt,
           "    #\n",
           "    # Estimates\n",
           "    #\n")
  #
  txt <-
    paste0(txt,
           "    y[(", n.out, " * i - ", n.out - 1,
           "):(", n.out, " * i)] ~ dmnorm.vcov(mean[(",
           n.out, " * i - ", n.out - 1, "):(", n.out,
           " * i)], S2[i, , ])\n")
  #
  txt <- paste0(txt, "  }\n")
  #
  if (multiarm) {
    #
    txt <-
      paste0(txt,
             "  #\n",
             "  # Three-arm studies\n",
             "  #\n")
    #
    txt <- paste0(txt, "  for (i in 1:(k - k2)) {\n")
    #
    idx <- rep(seq_len(n.out), 2)
    #
    txt <-
      paste0(txt,
             "    #\n",
             "    # Variances\n",
             "    #\n")
    #
    for (i in seq_len(2 * n.out)) {
      txt <-
        paste0(txt, "    S3[i, ", i, ", ", i, "] <- var", idx[i],
               "[k2 + 2 * i",
               if (i <= n.out) " - 1",
               "] + psi", idx[i], ".sq\n")
      #
      if (idx[i] == n.out)
        txt <- paste0(txt, "    #\n")
    }
    txt <-
      paste0(txt,
             "    # Covariances\n",
             "    #\n")
    #
    r <- 0
    #
    for (i in seq_len(n.out - 1)) {
      for (j in (i + 1):n.out) {
        r <- r + 1
        #
        txt <-
          paste0(txt,
                 "    S3[i, ", i, ", ", j, "] <- ",
                 "sqrt(S3[i, ", i, ", ", i, "]) * ",
                 "sqrt(S3[i, ", j, ", ", j, "]) * ",
                 "control", "[k2 + i, ", i, "] * ",
                 "control", "[k2 + i, ", j, "] * ",
                 "rho", r, "\n")
      }
    }
    #
    txt <- paste0(txt, "    #\n")
    #
    r <- 0
    #
    for (i in seq_len(n.out - 1)) {
      for (j in (i + 1):n.out) {
        r <- r + 1
        #
        txt <-
          paste0(txt,
                 "    S3[i, ", n.out + i, ", ", n.out + j, "] <- ",
                 "sqrt(S3[i, ", n.out + i, ", ", n.out + i, "]) * ",
                 "sqrt(S3[i, ", n.out + j, ", ", n.out + j, "]) * ",
                 "control", "[k2 + i, ", i, "] * ",
                 "control", "[k2 + i, ", j, "] * ",
                 "rho", r, "\n")
      }
    }
    #
    txt <- paste0(txt, "    #\n")
    #
    for (i in seq_len(n.out)) {
      txt <-
        paste0(txt,
               "    S3[i, ", i, ", ", i + n.out, "] <- 0.5 * ",
               "sqrt(S3[i, ", i, ", ", i, "]) * ",
               "sqrt(S3[i, ", i + n.out, ", ", i + n.out, "])\n")
    }
    #
    txt <- paste0(txt, "    #\n")
    #
    r <- 0
    #
    for (i in seq_len(n.out - 1)) {
      for (j in (i + 1):n.out) {
        r <- r + 1
        #
        txt <-
          paste0(txt,
                 "    S3[i, ", i, ", ", j + n.out, "] <- 0.5 * ",
                 "sqrt(S3[i, ", i, ", ", i, "]) * ",
                 "sqrt(S3[i, ", j + n.out, ", ", j + n.out, "]) * ",
                 "control", "[k2 + i, ", i, "] * ",
                 "control", "[k2 + i, ", j, "] * ",
                 "rho", r, "\n")
      }
    }
    #
    txt <- paste0(txt, "    #\n")
    #
    r <- 0
    #
    for (j in seq_len(n.out - 1)) {
      for (i in (j + 1):n.out){
        r <- r + 1
        #
        txt <-
          paste0(
            txt,
            "    S3[i, ", i, ", ", n.out + j, "] <- 0.5 * ",
            "sqrt(S3[i, ", i, ", ", i, "]) * ",
            "sqrt(S3[i, ", n.out + j, ", ", n.out + j, "]) * ",
            "control", "[k2 + i, ", j, "] * ",
            "control", "[k2 + i, ", i, "] * ",
            "rho", r, "\n")
      }
    }
    #
    txt <- paste0(txt, "    #\n")
    #
    for (i in seq_len(2 * n.out - 1)) {
      for (j in (i + 1):(2 * n.out)) {
        txt <-
          paste0(txt,
                 "    S3[i, ", j, ", ", i, "] <- S3[i, ", i, ", ", j, "]\n")
      }
    }
    #
    txt <-
      paste0(txt,
             "    #\n",
             "    # Estimates\n",
             "    #\n")
    #
    txt <-
      paste0(txt,
             "    y[(", n.out, " * k2 + ", 2 * n.out, " * i - ",
             2 * n.out - 1, "):(", n.out, " * k2 + ",
             2 * n.out, " * i)] ~\n      dmnorm.vcov(mean[(",
             n.out, " * k2 + ", 2 * n.out, " * i - ",
             2 * n.out - 1,
             "):(",
             n.out, " * k2 + ", 2 * n.out,
             " * i)], S3[i, , ])\n")
    #
    txt <- paste0(txt, "  }\n")
  }
  #
  txt <- paste0(txt, "  \n")
  #
  txt
}

code_means <- function(n.out, multiarm) {
  
  txt <-
    paste0(
      "  #\n",
      "  #\n",
      "  # (2) Parameterization of the means\n",
      "  #\n",
      "  #\n\n")
  #
  txt <-
    paste0(txt,
           "  #\n",
           "  # Two-arm studies\n",
           "  #\n")
  #
  txt <- paste0(txt, "  for (i in 1:k2) {\n")
  #
  for (i in seq_len(n.out)) {
    txt <-
      paste0(txt, "    mean[", n.out, " * i",
             if (i != n.out)
               paste0(" - ", n.out - i)
             else
               strrep(" ", nchar(n.out - 1) + 3),
             "] <- d", i, "[treat2[i]] - d", i, "[treat1[i]]\n")
  }
  #
  txt <- paste0(txt, "  }\n")
  #
  if (multiarm) {
    #
    txt <-
      paste0(txt,
             "  #\n",
             "  # Three-arm studies\n",
             "  #\n")
    #
    txt <- paste0(txt, "  for (i in 1:(k - k2)) {\n")
    #
    idx <- rep(seq_len(n.out), 2)
    #
    for (i in seq_len(2 * n.out)) {
      txt <-
        paste0(txt, "    mean[", n.out, " * k2 + ", 2 * n.out, " * i",
               if (i != 2 * n.out)
                 paste0(" - ", 2 * n.out - i)
               else
                 strrep(" ", nchar(2 * n.out - 1) + 3),
               "] <- d",
               idx[i], "[treat", 2 + (i > n.out), "[k2 + i]] - d",
               idx[i], "[treat1[k2 + i]]\n")
      #
      if (i != 2 * n.out & idx[i] == n.out)
        txt <- paste0(txt, "    #\n")
    }
    #
    txt <- paste0(txt, "  }\n")
  }
  txt
}

code_priors <- function(n.out, method,n.dom) {
  if (method == "standard")
    txt <- code_priors_standard(n.out)
  else
    txt <- code_priors_dumouchel(n.out,n.dom)
  #
  txt <- paste0(txt, "  #\n")
  #
  txt <- paste0(txt, code_priors_psi(n.out))
  #
  txt <- paste0(txt, "  #\n")
  #
  txt <- paste0(txt, code_priors_rho(n.out))
  #
  txt
}

code_priors_standard <- function(n.out) {
  
  txt <-
    paste0(
      "  #\n",
      "  #\n",
      "  # (3) Priors (standard model)\n",
      "  #\n",
      "  #\n\n")
  #
  txt <-
    paste0(txt,
           "  for (i in 1:(ref - 1)) {\n")
  #
  for (i in seq_len(n.out))
    txt <- paste0(txt, "    d", i, "[i] ~ dnorm(0, 1e-03)\n")
  #
  txt <- paste0(txt, "  }\n")
  #
  txt <- paste0(txt, "  #\n")
  for (i in seq_len(n.out))
    txt <- paste0(txt, "  d", i, "[ref] <- 0\n")
  txt <- paste0(txt, "  #\n")
  #
  txt <-
    paste0(txt,
           "  for (i in (ref + 1):n) {\n")
  #
  for (i in seq_len(n.out))
    txt <- paste0(txt, "    d", i, "[i] ~ dnorm(0, 1e-03)\n")
  #
  txt <- paste0(txt, "  }\n")
  #
  txt
}

code_priors_dumouchel <- function(n.out,n.dom) {
  
  txt <-
    paste0(
      "  #\n",
      "  #\n",
      "  # (4) Priors (DuMouchel model)\n",
      "  #\n",
      "  #\n\n")
  #
  if (is.null(n.dom)) {
    txt <-
      paste0(txt,
             "  for (i in 1:(ref - 1)) {\n",
             "    for (m in 1:", n.out, ") {\n",
             "      meand[m, i] <- alpha[i] + gamma[m]\n",
             "      d[m, i] ~ dnorm(meand[m, i], prec.exp)\n",
             "    }\n",
             "    #\n")
  }
  else {
    txt <-
      paste0(txt,
             "  for (i in 1:(ref - 1)) {\n",
             "    for (m in 1:", n.dom, ") {\n",
             "      meand[m, i] <- alpha1[i] + gamma1[m]\n",
             "      d[m, i] ~ dnorm(meand[m, i], prec.exp1)\n",
             "    }\n",
             "    for (l in ", n.dom+1, " : ", n.out, ") {\n",
             "      meand[l, i] <- alpha2[i] + gamma2[l]\n",
             "      d[l, i] ~ dnorm(meand[l, i], prec.exp2)\n",
             "    }\n",
             "    #\n")
  }
  #
  for (i in seq_len(n.out))
    txt <- paste0(txt, "    d", i, "[i] <- d[", i, ", i]\n")
  #
  txt <- paste0(txt, "  }\n")
  #
  txt <- paste0(txt, "  #\n")
  for (i in seq_len(n.out))
    txt <- paste0(txt, "  d", i, "[ref] <- 0\n")
  txt <- paste0(txt, "  #\n")
  #
  if (is.null(n.dom)) {
    txt <-
      paste0(txt,
             "  for (i in (ref + 1):n) {\n",
             "    for (m in 1:", n.out, ") {\n",
             "      meand[m, i] <- alpha[i] + gamma[m]\n",
             "      d[m, i] ~ dnorm(meand[m, i], prec.exp)\n",
             "    }\n",
             "    #\n")
  }
  else {
    txt <-
      paste0(txt,
             "  for (i in (ref + 1):n) {\n",
             "    for (m in 1:", n.dom, ") {\n",
             "      meand[m, i] <- alpha1[i] + gamma1[m]\n",
             "      d[m, i] ~ dnorm(meand[m, i], prec.exp1)\n",
             "    }\n",
             "    for (l in ", n.dom+1, " : ", n.out, ") {\n",
             "      meand[l, i] <- alpha2[i] + gamma2[l]\n",
             "      d[l, i] ~ dnorm(meand[l, i], prec.exp2)\n",
             "    }\n",
             "    #\n")
  }
  #
  for (i in seq_len(n.out))
    txt <- paste0(txt, "    d", i, "[i] <- d[", i, ", i]\n")
  #
  txt <- paste0(txt, "  }\n")
  #
  txt <- paste0(txt, "  #\n")
  #
  if (is.null(n.dom)) {
    txt <-
      paste0(txt,
             "  for (m in 1:", n.out, ") {\n",
             "  gamma[m] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  #\n")
  }
  else{
    txt <-
      paste0(txt,
             "  for (m in 1:", n.dom, ") {\n",
             "  gamma1[m] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  for (l in ", n.dom+1, " : ", n.out, ") {\n",
             "  gamma2[l] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  #\n")
  }
  #
  if (is.null(n.dom)) {
    txt <-
      paste0(txt,
             "  for (i in 1:(ref - 1)) {\n",
             "    alpha[i] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  #\n")
  }
  else {
    txt <-
      paste0(txt,
             "  for (i in 1:(ref - 1)) {\n",
             "    alpha1[i] ~ dnorm(0, 1e-03)\n",
             "    alpha2[i] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  #\n")
  }
  #
  if (is.null(n.dom)) {
    txt <-
      paste0(txt,
             "  for (i in (ref + 1):n) {\n",
             "    alpha[i] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  #\n")
  }
  else {
    txt <-
      paste0(txt,
             "  for (i in (ref + 1):n) {\n",
             "    alpha1[i] ~ dnorm(0, 1e-03)\n",
             "    alpha2[i] ~ dnorm(0, 1e-03)\n",
             "  }\n",
             "  #\n")
    
  }
  #
  if (is.null(n.dom)) {
    txt <-
      paste0(txt,
             "  prec.exp <- 1 / sigma.sq\n",
             "  sigma.sq <- sigma * sigma\n",
             "  sigma ~ dnorm(0, 1e-02)T(0, )\n")
  }
  else {
    txt <-
      paste0(txt,
             "  prec.exp1 <- 1 / sigma.sq1\n",
             "  sigma.sq1 <- sigma1 * sigma1\n",
             "  sigma1 ~ dnorm(0, 1e-02)T(0, )\n",
             "  prec.exp2 <- 1 / sigma.sq2\n",
             "  sigma.sq2 <- sigma2 * sigma2\n",
             "  sigma2 ~ dnorm(0, 1e-02)T(0, )\n"
      )
  }
  #
  txt
}

code_priors_psi <- function(n.out) {
  txt <- ""
  #
  for (i in seq_len(n.out))
    txt <- paste0(txt, "  psi", i, ".sq <- psi", i, " * psi", i, "\n")
  #
  txt <- paste0(txt, "  #\n")
  #
  for (i in seq_len(n.out))
    txt <- paste0(txt, "  psi", i, "  ~ dnorm(0, prec.psi", i, ")T(0, )\n")
  #
  txt
}

code_priors_rho <- function(n.out) {
  txt <- ""
  #
  for (i in seq_len(choose(n.out, 2)))
    txt <-
      paste0(txt, "  rho", i, " ~ dunif(lower.rho", i,
             ", upper.rho", i, ")\n")
  #
  txt
}

#' Forest plot for multivariate network meta-analysis results
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid graphics
#' system).
#' 
#' @param x An object of class \code{\link{mvnma}}.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param separate A logical indicating whether separate forest plots should be
#'   created for outcomes.
#' @param leftcols 	A character vector specifying (additional) columns to be
#'   printed on the left side of the forest plot or a logical value.
#' @param leftlabs 	A character vector specifying labels for (additional)
#'   columns on left side of the forest plot.
#' @param rightcols A character vector specifying (additional) columns to be 
#'   printed on the right side of the forest plot or a logical value.
#' @param rightlabs A character vector specifying labels for (additional)
#'   columns on right side of the forest plot. 
#' @param col.study The colour for treatment effect estimates and confidence
#'   interval limits. 
#' @param col.square The colour for squares reflecting the treatment effect
#'   estimates.
#' @param col.square.lines The colour for the outer lines of squares reflecting
#'   the treatment effect estimates.
#' @param col.subgroup The colour to print information on different outcomes.
#' @param squaresize The size of squares reflecting the treatment effect
#'   estimates (default: 0.7).
#' @param header.line A logical value indicating whether to print a header
#'   line (default: TRUE) or a character string ("both", "below", "").
#' @param text.outcome A text string printed before the name of the outcome.
#' @param \dots Additional arguments passed on to
#'   \code{\link[meta]{forest.meta}}.
#'
#' @return
#' A forest plot is shown in the active graphics window.
#' 
#' @keywords hplot
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Print multivariate network meta-analysis results
#' mvnma12
#' 
#' # Generate a forest plot with the results
#' forest(mvnma12)
#' 
#' # Generate a separate forest plot for each outcome
#' forest(mvnma12, separate = TRUE)
#' 
#' @method forest mvnma 
#' @export

forest.mvnma <- function(x, backtransf = FALSE,
                         #
                         separate = n_distinct(attr(x, "sm")) > 1,
                         #
                         leftcols = "studlab", leftlabs,
                         rightcols = c("effect", "ci"), rightlabs,
                         #
                         col.study = "black",
                         col.square = "black",
                         col.square.lines = "black",
                         col.subgroup = "black",
                         #
                         squaresize = 0.7,
                         header.line = TRUE, 
                         #
                         text.outcome = "Outcome: ",
                         #
                         ...) {
  
  chkclass(x, "mvnma")
  #
  chklogical(backtransf)
  chklogical(separate)
  chkchar(text.outcome, length = 1)
  #
  method.model <- attr(x, "method.model")
  reference.group <- attr(x, "reference.group")
  n.domain <- attr(x,"n.domain")
  sm <- attr(x, "sm")
  #
  x <- x[names(x) != "cor"]
  #
  if (method.model == "DM") {
    if (is.null(n.domain)) {
      x <- x[names(x) != "sigma"]
    }
    else{
      x <- x[!(names(x) %in% c("sigma1", "sigma2"))]
    }
  }
  #
  n.out <- length(x)
  #
  if (missing(leftlabs)) {
    if (separate)
      leftlabs <- "Treatment"
    else
      leftlabs <- paste0("Comparison with '", reference.group, "'")
  }
  #
  if (missing(rightlabs))
    rightlabs <- rep(NA, length(rightcols))
  
  # Get rid of warning "no visible binding for global variable"
  treat <- mean <- sd <- lower <- upper <- studlab <- NULL
  
  # Get estimates for each outcome
  #
  ests <- vector("list")
  #
  for (i in seq_len(n.out)) {
    ests[[i]] <- x[[i]]$basic_estimates
    ests[[i]]$treat <- row.names(ests[[i]])
    row.names(ests[[i]]) <- NULL
    #
    ests[[i]] %<>% select(treat, mean, sd, lower, upper)
    ests[[i]]$outcome <- attr(x, "names")[i]
  }
  
  # Create forest plot(s)
  #
  if (!separate) {
    dat <- bind_rows(ests)
    row.names(dat) <- NULL
    names(dat) <- c("studlab", "mean", "seTE", "lower", "upper", "outcome")
    #
    # Drop rows for reference group 
    #
    dat %<>% filter(studlab != reference.group)
    
    if (length(unique(sm)) == 1)
      sm <- unique(sm)
    else
      sm <- ""
    #
    m <- metagen(dat$mean, dat$seTE, sm = sm,
                 subgroup = paste0(text.outcome, dat$outcome),
                 backtransf = backtransf,
                 print.subgroup.name = FALSE,
                 studlab = dat$studlab,
                 common = FALSE, random = FALSE, hetstat = FALSE,
                 method.tau = "DL", method.tau.ci = "")
    #
    res <- forest(m,
                  header.line = header.line,
                  col.subgroup = "black",
                  leftcols = leftcols, 
                  leftlabs = leftlabs,
                  rightcols = rightcols,
                  rightlabs = rightlabs,
                  weight.study = "same",
                  col.study = col.study,
                  col.square = col.square,
                  col.square.lines = col.square.lines,
                  squaresize = squaresize,
                  #
                  calcwidth.subgroup = TRUE,
                  ...)
  }
  else {
    res <- vector("list", n.out)
    #
    for (i in seq_len(n.out)) {
      dat.i <- ests[[i]]
      row.names(dat.i) <- NULL
      names(dat.i) <- c("studlab", "mean", "seTE", "lower", "upper", "outcome")
      #
      # Drop rows for reference group 
      #
      dat.i %<>% filter(studlab != reference.group)
      #
      m.i <- metagen(dat.i$mean, dat.i$seTE, sm = sm[i],
                     backtransf = backtransf,
                     studlab = dat.i$studlab,
                     common = FALSE, random = FALSE, hetstat = FALSE,
                     method.tau = "DL", method.tau.ci = "")
      m.i$.text.details.methods <-
        paste0("Details:\n- Comparison with '", reference.group, "'")
      #
      res[[i]] <- forest(m.i,
                         header.line = header.line,
                         col.subgroup = "black",
                         leftcols = leftcols, 
                         leftlabs = leftlabs,
                         rightcols = rightcols,
                         rightlabs = rightlabs,
                         weight.study = "same",
                         col.study = col.study,
                         col.square = col.square,
                         col.square.lines = col.square.lines,
                         squaresize = squaresize,
                         #
                         smlab = paste0(text.outcome, unique(dat.i$outcome)),
                         #
                         details = TRUE,
                         calcwidth.subgroup = TRUE,
                         ...)
    }
  }
  #
  invisible(res)
}

#' Rank treatments across all outcomes using the VišeKriterijumska Optimizacija
#' I Kompromisno Rešenje (VIKOR) multi-criteria decision analysis method
#' 
#' @description
#' This function employs the VišeKriterijumska Optimizacija I Kompromisno
#' Rešenje (VIKOR) method to analyze all outcome-specific ranking lists.
#' It provides both an amalgamated ranking list and guidance on
#' which treatments correspond to the best compromise solutions.
#' 
#' @param x An object of class \code{\link{mvrank}} or a matrix.
#' @param weights Outcome weights. The weights should always sum to 1. If not
#'   then they are standardized. If NULL, the function will assume equal outcome
#'   weights. 
#' @param v A scalar from 0 to 1 interpreted as the weight of the decision
#'   making process. Following guidance from the multi-criteria decision
#'   analysis field it is set to 0.5.
#' @param digits A numeric specifying the number of digits to print the
#'   ranking matrix Q.
#' @param \dots Additional arguments (ignored).
#'
#' @details
#' This function takes a single mandatory argument, which is either an object
#' of class \code{\link{mvrank}} or a matrix. It then uses the multi-criteria
#' decision analysis method VišeKriterijumska Optimizacija I Kompromisno
#' Rešenje (VIKOR) to produce an amalgamated ranking list across
#' all outcomes (Opricovic & Tzeng, 2004).
#' 
#' The standard VIKOR approach is applied when the \code{method} argument is
#' set to \code{"sucra"} or \code{"pBV"} in \code{\link{mvrank}}.
#' 
#' A fuzzy VIKOR method (Opricovic, 2011) is applied when outcome-specific
#' rankings are expressed in terms of median ranks and 95\% credible intervals.
#' The latter is possible when the \code{\link{mvrank}} object is created with
#' \code{method = "ranks"}.
#' 
#' In both cases, the final ranking list is calculated based on treatments
#' common across all outcomes. Treatments not present across all outcomes are
#' excluded internally.
#' 
#' Using the argument 'weights' the users can specify the weight that each
#' outcome should have in the decision making process. For each outcome this
#' argument should have a value from 0 to 1 while the sum of all outcome
#' weights should be 1. If the sum of all weights is not 1, then these are
#' internally standardize to achieve this. The standardized weight values
#' are returned as a message to the user. Finally, if NULL then equal weights
#' are assumed across all outcomes.
#'
#' The argument 'v' specifies the weight of the decision making process.
#' The VIKOR method is a compromise programming approach that aims to balance
#' between each treatments overall and worst performance across all outcomes.
#' The balance between these two criteria is achieved using the parameter 'v'
#' which takes values from 0 to 1. Values close to 1 will give more weight to
#' the treatment's overall performance while values close to 0 will give more
#' weight to penalize the treatment's worst performance. The most common
#' choice of 'v' is typically 0.5 (default also here), thereby allowing for a
#' balanced decision making between treatment's overall and worst performance.
#' 
#' @return
#' The function returns a 'vikor' object. This consists of three ranking lists
#' which are the following:
#' \itemize{
#' \item A ranking list Q referring to the ranking when balancing both each
#'   treatment's overall and worst performance. This is the main ranking list
#'   of the method. 
#' \item A ranking list S referring to the ranking in terms of each treatment's
#'   overall performance.
#' \item A ranking list R referring to the ranking in terms of penalising each
#'   treatment's worst performance.
#' }
#' In addition to the ranking lists, the function also evaluates the necessary
#' conditions defined by the VIKOR method and returns a message indicating the
#' set of compromise solutions.
#'
#' @references
#' Opricovic S, Tzeng GH (2004):
#' Compromise solution by MCDM methods: A comparative analysis of VIKOR and
#' TOPSIS.
#' \emph{European Journal of Operational Research},
#' \bold{156}, 445--55
#' 
#' Opricovic S (2011):
#' Fuzzy VIKOR with an application to water resources planning. 
#' \emph{Expert Systems with Applications}, 
#' \bold{38}, 12983--90
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Print the results of a bivariate network meta-analysis
#' mvnma12
#' 
#' # Rank treatments using SUCRAs
#' ranks12 <- mvrank(mvnma12, method = "sucra",
#'   small.values = c("undes", "undes"))
#' ranks12
#' 
#' # Get the best compromise solution across the efficacy outcomes
#' vikor(ranks12)
#' 
#' # Use larger weight for response than remission
#' vikor(ranks12, weights = c(0.6, 0.3))
#'
#' @rdname vikor
#' @method vikor mvrank
#' @export

vikor.mvrank <- function(x, weights = NULL, v = 0.5, ...) {
  
  chkclass(x, "mvrank")
  #
  trts <- sort(attr(x, "common_trts"))
  outcomes <- names(x)
  
  if (attr(x, "method") %in% c("SUCRA", "pBV")) {
    # Get rid of warning "no visible binding for global variable"
    treatment <- NULL
    
    s <- vector("list")
    #
    x.common <- attr(x,"ranks.common")
    #
    for (i in seq_len(length(x))) {
      dat.i <- x.common[[i]] %>% 
        filter(treatment %in% trts) %>% 
        arrange(treatment)
      #
      s[[i]] <- as.data.frame(dat.i[, 2])
      #
      names(s[[i]]) <- paste("ranks", i, sep = "_")
    }
    #
    rankings <- bind_cols(s)
    #
    row.names(rankings) <- dat.i$treatment
    names(rankings) <- outcomes
    #
    res <- vikor_internal(rankings, weights = weights, v = v)
  }
  else {
    decision <- performance_fuzzy(x,trts)
    res <- fuzzy_vikor_internal(decision, weights = weights,v = v )
  }
  #
  class(res) <- c("vikor", class(res))
  attr(res, "ranking.method") <- attr(x, "method")
  #
  res
}


#' @rdname vikor
#' @method vikor matrix
#' @export

vikor.matrix <- function(x, weights = NULL, v = 0.5, ...) {
  
  chkclass(x, "matrix")
  #
  res <- vikor_internal(x, weights = weights, v = v)
  #
  res
}


#' @rdname vikor
#' @export vikor

vikor <- function(x, ...)
  UseMethod("vikor")


#' @rdname vikor
#' @method print vikor
#' @export

print.vikor <- function(x, digits = 4, ...) {
  
  chkclass(x, "vikor")
  #
  chknumeric(digits, min = 0, length = 1)
  
  Q <- x %>% select(Q)
  S <- x %>% select(S)
  R <- x %>% select(R)
  #
  trts <- row.names(Q)
  #
  DQ <- 1 / (length(trts) - 1)
  
  cond1 <- Q$Q[2] - Q$Q[1] >= DQ
  #
  cond2_1 <- isTRUE(row.names(Q)[1] == row.names(S)[1])
  cond2_2 <- isTRUE(row.names(Q)[1] == row.names(R)[1])
  #
  cond2 <- isTRUE(cond2_1 & cond2_2)
  #
  if (cond1 & cond2) {
    solution <- row.names(Q)[1]
    #
    txt <- paste("The compromise treatment across all outcomes is:", solution)
  }
  else if ((cond1) & (!cond2)) {
    solution <- paste(row.names(Q)[1:2], collapse = ", ")
    #
    txt <- paste("The compromise set of treatments across all outcomes are:",
                 solution)
  }
  else if (!cond1) {
    compr <- Q$Q - Q$Q[1] < DQ
    #
    E <- which(compr)
    #
    solution <- paste(row.names(Q)[E], collapse = ", ")
    #
    txt <- paste("The compromise set of treatments across all outcomes are:",
                 solution)
  }
  else if (!cond1 & !cond2)
    txt <- paste("No compromise solution was identified. Please consider",
                 "different outcome weights.")
  
  res_mat <- cbind(Q, S, R)
  
  if (attr(x, "ranking.method") %in% c("SUCRA", "pBV"))
    cat("VIKOR results\n\n")
  else
    cat("Fuzzy VIKOR results\n\n")
  #
  prmatrix(round(res_mat, digits = digits), quote = FALSE, right = TRUE)
  #
  cat(paste0("\n", txt, "\n"))
  #
  invisible(NULL)
}

vikor_internal <- function(x, weights, v) {
  
  minmax <- rep("max", ncol(x))
  
  # Assume equal outcome weights if argument 'weights' is NULL
  #
  if (is.null(weights))
    weights <- rep(1 /ncol(x), ncol(x))
  else if ((!is.null(weights) & length(weights) != ncol(x)))
    stop("Please provide weights for all outcomes.",
         call. = FALSE)
  #
  if (!is.null(weights) && sum(weights) != 1) {
    weights <- round(weights / sum(weights), digits = 2)
    #
    warning("Weights should always sum up to 1. To do so the given weights ",
            "are now standardized and the new weights are: ",
            paste(weights, collapse = ", "))
  }
  #
  critno <- ncol(x)
  #
  pos.sol <- rep(1, critno)
  neg.sol <- rep(0, critno) 
  #
  dist <- matrix(ncol = critno, nrow = nrow(x),
                 dimnames = list(row.names(x), colnames(x)))
  #
  for (i in seq_len(critno))
    dist[, i] <- (pos.sol[i] - x[, i]) / (pos.sol[i] - neg.sol[i])
  
  altno <- nrow(x)
  wnm <- t(t(dist) * weights)
  #
  Q <- R <- S <- vector("numeric", altno)
  #
  for (i in seq_len(altno)) {
    R[i] <- max(wnm[i, ])
    S[i] <- sum(wnm[i, ])
  }
  
  for (i in seq_len(altno)) {
    Q[i] <- (v * (S[i] - min(S)) / (max(S) - min(S))) + 
      ((1 - v) * (R[i] - min(R)) / (max(R) - min(R)))
  }
  #
  res <- data.frame(Q, S, R, row.names = row.names(x)) %>% arrange(Q)
  
  #
  #class(res) <- c("vikor", class(res))
  attr(res, "performance.table") <- x
  #
  res
}


performance_fuzzy <- function(x, trts, ...) {
  
  # Get rid of warning "no visible binding for global variable"
  #
  treatment <- lower.CrI <- median_rank <- upper.CrI <- NULL
  
  class(x) <- "list"
  
  for (i in seq_along(x)) {
    x[[i]] %<>%
      filter(treatment %in% trts) %>% 
      arrange(treatment) %>% 
      select(treatment, lower.CrI, median_rank, upper.CrI)
    #
    row.names(x[[i]]) <- x[[i]]$treatment
    #
    x[[i]] %<>% select(-treatment)
    #
    names(x[[i]]) <- paste(names(x[[i]]), i, sep = ".")
  }
  
  y <- bind_cols(x)    
  names(y) <- seq(1:ncol(y))
  y <- as.matrix(y)
  #
  y
}

fuzzy_vikor_internal <- function(x, weights, v = 0.5) {
  
  # Adjust weights
  
  # Number of outcomes
  n.out <- ncol(x) / 3
  #
  if (is.null(weights))
    weights <- rep(1 / n.out, n.out)
  else if ((!is.null(weights) & length(weights) != n.out))
    stop("Please provide weights for all outcomes.",
         call. = FALSE)
  #
  if (!is.null(weights) && sum(weights) != 1) {
    weights <- round(weights / sum(weights), digits = 2)
    #
    warning("Weights should always sum up to 1. To do so the given weights ",
            "are now standardized and the new weights are: ",
            paste(weights, collapse = ", "))
  }
  #
  # Create weights fuzzy weights
  #
  weights_fuzzy <- rep(c(weights / 3), each = 3)
  #
  pos.sol <- rep(1, length(weights_fuzzy))
  neg.sol <- rep(nrow(x), length(weights_fuzzy))
  
  # 1. S and R index
  
  # Calculate differences
  d <- matrix(nrow = nrow(x), ncol = ncol(x))
  for (i in seq(1, ncol(x), 3)) {
    # denominator = neg.sol[i + 2] - posI[i]
    d[, i] <- (x[, i] -pos.sol[i + 2]) / (neg.sol[i] - pos.sol[i])
    d[, i + 1] <- (x[, i + 1] - pos.sol[i + 1]) / (neg.sol[i] - pos.sol[i])
    d[, i + 2] <- (x[, i + 2] - pos.sol[i]) / (neg.sol[i] - pos.sol[i])
  }
  
  W <- diag(weights_fuzzy)
  
  NW <- d %*% W
  
  S <- matrix(nrow = nrow(x), ncol = 3)
  R <- matrix(nrow = nrow(x), ncol = 3)
  
  S[, 1] <- apply(NW[,seq(1, ncol(x), 3)], 1, sum)
  S[, 2] <- apply(NW[,seq(2, ncol(x), 3)], 1, sum)
  S[, 3] <- apply(NW[,seq(3, ncol(x), 3)], 1, sum)
  
  R[, 1] <- apply(NW[,seq(1, ncol(x), 3)], 1, max)
  R[, 2] <- apply(NW[,seq(2, ncol(x), 3)], 1, max)
  R[, 3] <- apply(NW[,seq(3, ncol(x), 3)], 1, max)
  
  # 2. Q index
  
  Q1 <- matrix(nrow = nrow(x), ncol = 3)
  Q2 <- matrix(nrow = nrow(x), ncol = 3)
  
  denominatorS <- max(S[, 3]) - min(S[, 1])
  Q1[, 1] <- (S[, 1] - min(S[, 3])) / denominatorS
  Q1[, 2] <- (S[, 2] - min(S[, 2])) / denominatorS
  Q1[, 3] <- (S[, 3] - min(S[, 1])) / denominatorS
  
  denominatorR = max(R[, 3]) - min(R[, 1])
  Q2[, 1] <- (R[, 1] - min(R[, 3])) / denominatorR
  Q2[, 2] <- (R[, 2] - min(R[, 2])) / denominatorR
  Q2[, 3] <- (R[, 3] - min(R[, 1])) / denominatorR
  
  if (v == 1)
    Q <- Q1
  else if (v == 0)
    Q <- Q2
  else
    Q <- v * Q1 + (1 - v) * Q2
  
  # 3. De-fuzzification using the method by Opricovic
  
  Def_S <- (S[, 1] + S[, 2] * 2 + S[, 3]) / 4
  Def_R <- (R[, 1] + R[, 2] * 2 + R[, 3]) / 4
  Def_Q <- (Q[, 1] + Q[, 2] * 2 + Q[, 3]) / 4
  
  # 4. Return results
  
  res <- data.frame(Q = Def_Q, S = Def_S, R = Def_R,
                    row.names = row.names(x)) %>% arrange(Q)
  #
  #class(res) <- c("vikor", class(res))
  attr(res, "performance.table") <- x
  attr(res, "fuzzy.weights") <- weights_fuzzy
  #
  res
}

#' Line chart showing the VIKOR results
#'
#' @description
#' A line chart showing the results of VišeKriterijumska Optimizacija I
#' Kompromisno Rešenje (VIKOR) across the three metrics Q, S and R for each
#' treatment. Within each metric, lower values represent better treatment
#' performance.
#' 
#' @param x An object of class \code{\link{vikor}}.
#' @param sort A character specifying the order of treatments on the x-axis.
#'   By default, the order is according to the Q-metric (\code{"Q"}). Ordering
#'   treatments according to the S (\code{"S"}) and R (\code{"R"}) metrics is
#'   also possible.
#' @param exclude A character specifying a metric that will not be displayed in
#'   the graph. By default, all metrics are displayed (\code{"none"}).
#'   Alternative options are to exclude either the Q-metric (\code{"Q"}),
#'   the S-metric (\code{"S"}), or the R-metric (\code{"R"}).
#' @param n A numeric value indicating the number of treatments to be plotted
#'   in the graph. By default, all treatments are displayed. If specified,
#'   only the first \code{"n"} treatments according to the hierarchy specified
#'   from \code{"sort"} argument are plotted.
#' @param linewidth A numeric value specifying the width of the lines
#'   (default: 1.1).
#' @param size A numeric value specifying the size of the points
#'   (default: 2).
#' @param \dots Additional arguments passed to \code{\link[ggplot2]{ggplot}}
#'   function.
#'
#' @return
#' A \code{ggplot} object.
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Rank treatments using SUCRAs (default)
#' ranks12 <- mvrank(mvnma12, small.values = c("undes", "undes"))
#' ranks12
#' 
#' # Get the best compromise solution across the efficacy outcomes
#' vk12 <- vikor(ranks12)
#' 
#' # Visualize the results with default settings
#' linechart(vk12)
#' 
#' # Sort by the "R" metric
#' linechart(vk12, sort = "R")
#' 
#' # Sort by the "R" metric and include only the first 3 treatments
#' linechart(vk12, sort = "R", n = 3)
#' 
#' # Exclude the "R" metric
#' #
#' linechart(vk12, exclude = "R")
#' 
#' @export linechart

linechart <- function(x, 
                      sort = "Q",
                      exclude = "none",
                      n = nrow(x),
                      linewidth = 1.1,
                      size = 2,
                      ...) {
  
  chkclass(x, "vikor")
  #
  sort <- setchar(sort, c("Q" , "S", "R"))
  exclude <- setchar(exclude, val = c("none", "Q", "S", "R"))
  #
  chknumeric(n, min = 1, max = nrow(x), length = 1)
  chknumeric(linewidth, min = 0, zero = TRUE, length = 1)
  chknumeric(size, min = 0, zero = TRUE, length = 1)
  #
  if (sort == exclude)
    stop("The excluded list cannot coincide with the list used for sorting.",
         call. = FALSE)
  #
  chknumeric(n, min = 0)
  
  treat <- rep(row.names(x), 3)
  values <- c(x$Q, x$S, x$R)
  
  type <- rep(c("Q", "S", "R"), each = length(treat) / 3)
  
  dat <- data.frame(treat, type, values)
  #
  dat.sort <- dat %>% 
    filter(type == sort) %>% 
    arrange((values))
  
  dat.sort <- dat.sort[seq_len(n), ]
  sort.treat <- dat.sort$treat
  levels.list <- c("Q", "S", "R")
  
  if (exclude == "none") {
    dat <- dat %>% 
      filter(treat %in% sort.treat) %>% 
      mutate(type = factor(type, levels = levels.list)) %>% 
      mutate(treat = factor(treat, levels = sort.treat))
  }
  else {
    l <- which(levels.list == exclude)
    #
    levels.list <- levels.list[-l]
    #
    dat <- dat %>%
      filter(type != exclude) %>% 
      filter(treat %in% sort.treat) %>% 
      mutate(type = factor(type, levels = levels.list)) %>% 
      mutate(treat = factor(treat, levels = sort.treat))
  }
  
  p <- ggplot(dat, aes(x = treat, y = values, color = type, group = type)) +
    geom_line(linewidth = linewidth) +
    geom_point(size = size) +
    theme_minimal() +
    xlab("") +
    ylab("Value") +
    ylim(c(0, 1)) +
    guides(color = guide_legend(title = "Metric"))
  #
  attr(p, "data") <- dat
  #
  p
}

catch <- function(argname, matchcall, data, encl)
  eval(matchcall[[match(argname, names(matchcall))]], data, enclos = encl)

multi_arm <- function(data) {
  
  # Get rid of warning "no visible binding for global variable"
  studlab <- treat2 <- NULL
  
  studies <- unique(data$studlab)    
  #
  r <- t <- E <- vector("list")
  
  for (i in seq_along(studies)) {
    r[[i]] <- data %>% filter(studlab == studies[i])
    #
    if (nrow(r[[i]]) > 2) {
      t[[i]] <- as.data.frame(table(r[[i]]$treat2))
      E[[i]] <- t[[i]]$Var1[which(t[[i]]$Freq == max(t[[i]]$Freq))]
      r[[i]] %<>% filter(treat2 == E[[i]])
    }
  }
  #
  # df harmonized in terms of treat2 
  #
  res <- list.rbind(r)
  #
  res
}

create_T <- function(data, max.arms) {
  # Get rid of warning "no visible binding for global variable"
  studlab <- NULL
  #
  studies <- unique(data$studlab)  
  #
  res <- matrix(NA, nrow = length(studies), ncol = max.arms)
  #
  for (i in seq_along(studies)) {
    dat.i <- data %>% filter(studlab == studies[i])
    #
    trts.i <- c(unique(dat.i$id2), unique(dat.i$id1))
    #
    res[i, seq_along(trts.i)] <- trts.i
  }
  #
  rownames(res) <- studies
  colnames(res) <- paste0("id", seq_len(ncol(res)))
  #
  res
}

'%!in%' <- function(x, y)
  !('%in%'(x, y))


#
# Helpers for mvdata()
#

create_data <- function(p, ...) {
  
  if (!(any(class(p) == "list")))
    stop("Argument 'p' must be a list of pairwise objects.", 
         call. = FALSE)
  
  # Get rid of warning "no visible binding for global variable"
  studlab <- TE <- seTE <- treat1 <- treat2 <- outcome <- n.arms <- NULL
  #
  dat1 <- dat2 <- studies <- vector("list")
  #
  n.out <- length(p)
  #
  for (i in 1:n.out) {
    p[[i]] %<>% select(studlab, TE, seTE, treat1, treat2)
    p[[i]] <- multi_arm(p[[i]])
    p[[i]]$outcome <- i
    p[[i]] <- add_arms(p[[i]])
  }
  #
  # Combine all pairwise objects
  #
  comb_p <- bind_rows(p)
  #
  # Add missing comparisons for three-arm studies
  #
  twoarm <- comb_p %>%
    filter(n.arms == 2) %>%
    select(studlab) %>% unique() %>% pull()
  #
  threearm <- comb_p %>%
    filter(n.arms == 3) %>%
    select(studlab) %>% unique() %>% pull()
  #
  both <- twoarm[twoarm %in% threearm]
  #
  for (i in both) {
    drugs.i <- comb_p %>% filter(studlab == i, n.arms == 3) %>%
      select(studlab, treat1, treat2) %>% distinct()
    #
    dat2.i <- comb_p %>% filter(studlab == i, n.arms == 2)
    #
    for (j in unique(dat2.i$outcome)) {
      drugs.ij <- drugs.i %>% mutate(outcome = j, n.arms = 3)
      dat2.ij <- dat2.i %>% filter(outcome == j)
      #
      drop.j <- vector()
      for (k in seq_len(nrow(drugs.ij))) {
        drop.j[k] <- any(dat2.ij$studlab == i & dat2.ij$outcome == j &
                           dat2.ij$treat1 == drugs.ij$treat1[k] &
                           dat2.ij$treat2 == drugs.ij$treat2[k])
      }
      #
      comb_p <- bind_rows(comb_p, drugs.ij %>% filter(!drop.j))
      comb_p$n.arms[comb_p$studlab == i & comb_p$outcome == j] <- 3
    }
  }
  
  for (i in seq_len(n.out)) {
    dat1[[i]] <- comb_p %>% filter(outcome == i)
    dat2[[i]] <- comb_p %>% filter(outcome != i) 
    #
    dat2[[i]]$TE <- dat2[[i]]$seTE <- NA
    dat2[[i]]$new_outcome <- i
    #
    studies[[i]] <- unique(which(dat2[[i]]$studlab %!in% dat1[[i]]$studlab))
    #
    if (length(studies[[i]]) > 0) {
      dat2[[i]] <- dat2[[i]][studies[[i]], ]
      dat2[[i]]$outcome <- dat2[[i]]$new_outcome
      dat2[[i]]$new_outcome <- NULL
      #
      row.names(dat2[[i]]) <- NULL
    }
    else
      dat2[[i]] <- list()
  }
  #
  if (length(dat2) != 0) {
    dat2 <- list.rbind(dat2)
    #
    comb_p <- rbind.data.frame(comb_p, dat2)
    comb_p <- comb_p %>% arrange(outcome)
    #
    row.names(comb_p) <- NULL
  }
  else
    comb_p %<>% arrange(outcome)
  #
  comb_p %<>% distinct()
  #
  res <- comb_p %>%
    distinct() %>%
    arrange(outcome) %>%
    group_by(studlab) %>%
    arrange(desc(treat1), .by_group = TRUE) %>%
    arrange(n.arms)
  #
  res <- as.data.frame(res)
  rownames(res) <- seq_len(nrow(res))
  #
  res <- add_ids(res)
  #
  res
}

add_arms <- function(x, ...) {
  # Get rid of warning "no visible binding for global variable"
  studlab <- NULL
  #
  studies <- unique(x$studlab)
  #
  res <- vector("list")
  #
  for (i in seq_along(studies)) {
    res[[i]] <- x %>% filter(studlab == studies[i])
    #
    res[[i]]$n.arms <- length(unique(c(res[[i]]$treat1, res[[i]]$treat2)))
  }
  #
  res <- list.rbind(res)  
  #
  res
}

add_ids <- function(data) {
  all_treats <- unique(c(data$treat1, data$treat2))  
  #
  levels_treats <- as.data.frame(levels(as.factor(all_treats)))
  names(levels_treats) <- c("treat")
  levels_treats$level <- 1:nrow(levels_treats)  
  #
  data$id1 <- NA
  data$id2 <- NA
  #
  for (i in 1:nrow(data)) {
    for (j in 1:nrow(levels_treats)) {
      if (data$treat1[i] == levels_treats$treat[j]) {
        data$id1[i] = levels_treats$level[j]
      }
      #
      if (data$treat2[i] == levels_treats$treat[j]) {
        data$id2[i] = levels_treats$level[j]
      }
    }
  }  
  #
  data
}

make_jags_data <- function(dat) {
  
  # Get rid of warning "no visible binding for global variable"
  outcome <- seTE <- studlab <- TE <- var <- NULL
  
  # Number of outcomes
  n.out <- length(unique(dat$outcome))
  
  # Number of studies  
  k <- length(unique(dat$studlab))    
  
  # Treatments
  trts <- sort(unique(c(dat$treat1, dat$treat2)))
  
  # Number of treatments
  n <- length(trts)
  
  # Arms per study
  arm_data <- dat[!duplicated(dat$studlab), ]
  
  # Number of two-arm studies
  k2 <- sum(arm_data$n.arms == 2, na.rm = TRUE)
  
  treat_data <- create_T(dat, max.arms = max(arm_data$n.arms))
  
  # Extract vector with treatment effects
  #
  y <- dat$TE
  y <- ifelse(is.na(y), 0, y)
  #
  trts.list <- vector("list", n.out)
  dat_var <- NULL
  #
  for (i in seq_len(n.out)) {
    dat_var_i <- subset(dat, outcome == i)
    #
    if (is.null(dat_var))
      dat_var <- data.frame(studlab = dat_var_i$studlab)
    #
    dat_var_i_complete <- subset(dat_var_i, complete.cases(TE))
    trts.list[[i]] <- with(dat_var_i_complete, sort(unique(c(treat1, treat2))))
    #
    dat_var_i %<>% mutate(var = seTE^2) %>% select(var)
    #
    names(dat_var_i) <- paste0("var", i)
    #
    dat_var <- cbind(dat_var, dat_var_i)
  }
  #
  res <- list(y = y, var = dat_var, T = treat_data,
              k = k, k2 = k2, n = n, 
              trts = trts, trts.list = trts.list)
  #
  res
}

is.list.pairwise <- function(p, ...) {
  all_class <- sapply(p, class)
  #
  check <- vector()
  #
  for (i in 1:ncol(all_class))
    check[i] <- isTRUE("pairwise" %in% all_class[, i])  
  #
  pair <- ifelse(sum(check) >= 2, "pairwise", NA)
  #
  pair
}

gather_results <- function(x, outcomes, trts, reference.group,
                           level, trts.list, method,n.domain, ...) {
  res <- as.data.frame(x$BUGSoutput$summary)
  samples <- x$BUGSoutput$sims.list
  #
  n.chain <- x$BUGSoutput$n.chain
  n.thin <- x$BUGSoutput$n.thin
  n.burnin <- x$BUGSoutput$n.burnin
  #
  ref <- which(trts == reference.group)
  rnames <- rownames(res)
  #
  n.out <- length(outcomes)
  #
  lower.level <- (1 - level) / 2
  upper.level <- 1 - (1 - level) / 2
  #
  # Make the elements of the list "trts.list" to be every possible treatment if
  # argument 'method == "DM"'
  if (method == "DM") {
    for (i in seq_along(trts.list))
      trts.list[[i]] <- sort(trts)
  }
  #
  basic <- dat_treat <- psi <- rho <- d <-
    TE.random <- seTE.random <- lower.random <- upper.random <-
    vector("list")
  # Get rid of warning "no visible binding for global variable"
  sd <- Rhat <- n.eff <- lower <- upper <- out_num <- NULL
  #
  for (i in seq_len(n.out)) {
    d.i <- paste0("d", i)
    #
    # Samples
    #
    d[[i]] <- samples[[d.i]]
    colnames(d[[i]]) <- trts
    rownames(d[[i]]) <- seq_len(nrow(d[[i]]))
    #
    d[[i]] <- as.data.frame(d[[i]]) %>% 
      select(any_of(trts.list[[i]]))
    #
    # Results for basic parameters
    #
    basic[[i]] <- res %>%
      filter(grepl(paste0(d.i, "["), rnames, fixed = TRUE)) %>%
      mutate(lower = NA, upper = NA)
    #
    row.names(basic[[i]]) <- trts
    basic[[i]] <- basic[[i]][which(row.names(basic[[i]]) %in% trts.list[[i]]), ]
    ##
    for (j in rownames(basic[[i]])) {
      basic[[i]][j, c("lower", "upper")] <-
        quantile(d[[i]][[j]], probs = c(lower.level, upper.level), na.rm = TRUE)
    }
    #
    basic[[i]] %<>% select(mean, sd, lower, upper, Rhat, n.eff)
    #
    basic[[i]][reference.group, ] <- NA
    #
    # psi's (similar to tau)
    #
    psi[[i]] <- res %>% filter(grepl("psi", rnames)) %>%
      select(mean, sd, "2.5%", "97.5%", Rhat, n.eff) %>%
      rename(psi = mean, lower = "2.5%", upper = "97.5%")
    #
    # rho
    #
    rho[[i]] <- res %>% filter(grepl("rho", rnames))
    # ensure the order of outcomes
    rho[[i]]$out_num <- as.numeric(gsub("rho", "", rownames(rho[[i]])))
    #
    rho[[i]] %<>% arrange(out_num)
    rho[[i]]$out_num <- NULL
    
    #
    # Matrices with all results
    #
    dmat.i <- d[[i]]
    #
    TE.random.i <- seTE.random.i <-
      lower.random.i <- upper.random.i <-
      matrix(NA, nrow = ncol(dmat.i), ncol = ncol(dmat.i),
             dimnames = list(trts.list[[i]], trts.list[[i]]))
    #
    for (j in seq_len(ncol(dmat.i)))
      for (k in seq_len(ncol(dmat.i)))
        if (j != k)
          TE.random.i[j, k] <- mean(dmat.i[, j] - dmat.i[, k])
    #
    for (j in seq_len(ncol(dmat.i)))
      for (k in seq_len(ncol(dmat.i)))
        if (j != k)
          seTE.random.i[j, k] <- sd(dmat.i[, j] - dmat.i[, k])
    #
    for (j in seq_len(ncol(dmat.i)))
      for (k in seq_len(ncol(dmat.i)))
        if (j != k)
          lower.random.i[j, k] <- quantile(dmat.i[, j] - dmat.i[, k],
                                           lower.level)
    #
    for (j in seq_len(ncol(dmat.i)))
      for (k in seq_len(ncol(dmat.i)))
        if (j != k)
          upper.random.i[j, k] <- quantile(dmat.i[, j] - dmat.i[, k],
                                           upper.level)
    #
    diag(TE.random.i) <- diag(seTE.random.i) <-
      diag(lower.random.i) <- diag(upper.random.i) <- 0
    #
    TE.random[[i]] <- TE.random.i
    seTE.random[[i]] <- seTE.random.i
    lower.random[[i]] <- lower.random.i
    upper.random[[i]] <- upper.random.i
  }
  #
  # Prepare output
  #
  cor <- rho[[1]]
  #
  cor %<>% select(mean, sd, "2.5%", "97.5%", Rhat, n.eff) %>%
    rename(lower = "2.5%", upper = "97.5%")
  #
  psi <- psi[[1]]
  row.names(psi) <- outcomes
  #
  if (method == "DM")
    if(is.null(n.domain)){
    sigma <- res %>% filter(grepl("sigma", rnames))
    }else{
      sigma1 <- res %>% filter(grepl("sigma1", rnames))
      sigma2 <- res %>% filter(grepl("sigma2", rnames))
    }
  #
  # Create row.names for cor
  #
  r1 <- t(combn(seq_along(outcomes), 2))
  r.names <- vector("numeric", nrow(cor))
  #
  for (i in seq_along(r.names)) {
    for (j in seq_len(ncol(r1))) {
      r.names[i] <- paste(outcomes[r1[i, 1]], outcomes[r1[i, 2]], sep = "/")    
    }
  }
  #
  row.names(cor) <- r.names
  #
  out1 <- list(basic_estimates = basic[[1]],
               heterogeneity = psi[1, ],
               TE.random = TE.random[[1]],
               seTE.random = seTE.random[[1]],
               lower.random = lower.random[[1]],
               upper.random = upper.random[[1]],
               samples = d[[1]])
  #
  out2 <- list(basic_estimates = basic[[2]],
               heterogeneity = psi[2, ],
               TE.random = TE.random[[2]],
               seTE.random = seTE.random[[2]],
               lower.random = lower.random[[2]],
               upper.random = upper.random[[2]],
               samples = d[[2]])
  #
  res <- list(out1, out2)
  names(res) <- c(outcomes[1], outcomes[2])
  #
  if (n.out >= 3) {
    out3 <- list(basic_estimates = basic[[3]],
                 heterogeneity = psi[3, ],
                 TE.random = TE.random[[3]],
                 seTE.random = seTE.random[[3]],
                 lower.random = lower.random[[3]],
                 upper.random = upper.random[[3]],
                 samples = d[[3]])
    #
    res[[3]] <- out3
    names(res)[3] <- outcomes[3]
  }
  #
  if (n.out >= 4) {
    out4 <- list(basic_estimates = basic[[4]],
                 heterogeneity = psi[4, ],
                 TE.random = TE.random[[4]],
                 seTE.random = seTE.random[[4]],
                 lower.random = lower.random[[4]],
                 upper.random = upper.random[[4]],
                 samples = d[[4]])
    #
    res[[4]] <- out4
    names(res)[4] <- outcomes[4]
  }
  #
  if (n.out >= 5) {
    out5 <- list(basic_estimates = basic[[5]],
                 heterogeneity = psi[5, ],
                 TE.random = TE.random[[5]],
                 seTE.random = seTE.random[[5]],
                 lower.random = lower.random[[5]],
                 upper.random = upper.random[[5]],
                 samples = d[[5]])
    #
    res[[5]] <- out5
    names(res)[5] <- outcomes[5]
  }
  #
  res[[length(res) + 1]] <- cor
  names(res)[length(res)] <- "cor"
  #
  if (method == "DM") {
    if(is.null(n.domain)){
    res[[length(res) + 1]] <- sigma
    names(res)[length(res)] <- "sigma"
    }else{
      res[[length(res) + 1]] <- sigma1
      names(res)[length(res)] <- "sigma1"
      #
      res[[length(res) + 1]] <- sigma2
      names(res)[length(res)] <- "sigma2"
    }
  }
  #
  res
}

setchar <- function(x, val, text, list = FALSE, name = NULL,
                    stop.at.error = TRUE, addtext = "",
                    return.NULL = TRUE, nchar.equal = FALSE,
                    setNA = FALSE, pre = "") {
  val <- unique(val)
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  nval <- length(val)
  #
  if (is.numeric(x)) {
    numeric.x <- TRUE
    idx <- x
    idx[idx < 1] <- NA
    idx[idx >= nval + 1] <- NA
  }
  else {
    numeric.x <- FALSE
    #
    if (length(unique(tolower(x))) != length(unique(x)) |
        length(unique(tolower(val))) != length(unique(val)))
      idx <- charmatch(x, val, nomatch = NA)
    else
      idx <- charmatch(tolower(x), tolower(val), nomatch = NA)
  }
  #
  if ((anyNA(idx) || any(idx == 0)) && !setNA) {
    if (list)
      first <- "List element '"
    else
      first <- "Argument '"
    #
    if (missing(text)) {
      if (numeric.x) {
        if (nval == 1)
          vlist <- "1"
        else if (nval == 2)
          vlist <- "1 or 2"
        else
          vlist <- paste("between 1 and", nval)
      }
      else {
        if (nval == 1)
          vlist <- paste0('"', val, '"')
        else if (nval == 2)
          vlist <- paste0('"', val, '"', collapse = " or ")
        else
          vlist <- paste0(paste0('"', val[-nval], '"', collapse = ", "),
                          ', or ', '"', val[nval], '"')
      }
      #
      if (stop.at.error)
        stop(first, name, "' must be ", pre,
             vlist, addtext, ".", call. = FALSE)
      else {
        if (return.NULL)
          return(NULL)
        else
          return(x)
      }
    }
    else {
      if (stop.at.error)
        stop(first, name, "' ", text, ".", call. = FALSE)
      else {
        if (return.NULL)
          return(NULL)
        else
          return(x)
      }
    }
  }
  #
  if (is.null(x))
    return(NULL)
  else
    res <- val[idx]
  #
  if (nchar.equal && nchar(res) != nchar(x))
    res <- x
  #
  res
}

chkclass <- function(x, class, name = NULL) {
  #
  # Check class of R object
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  n.class <- length(class)
  if (n.class == 1)
    text.class <- paste0('"', class, '"')
  else if (n.class == 2)
    text.class <- paste0('"', class, '"', collapse = " or ")
  else
    text.class <- paste0(paste0('"', class[-n.class], '"', collapse = ", "),
                         ', or ', '"', class[n.class], '"')
  #
  if (!inherits(x, class))
    stop("Argument '", name, "' must be an object of class ", text.class, ".",
         call. = FALSE)
  #
  invisible(NULL)
}

chkchar <- function(x, length = 0, name = NULL, nchar = NULL, single = FALSE,
                    NULL.ok = FALSE) {
  if (is.null(x) & NULL.ok)
    return(invisible(NULL))
  #
  if (!missing(single) && single)
    length <- 1
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (length && length(x) != length) {
    if (length == 1)
      stop("Argument '", name, "' must be a character string.",
           call. = FALSE)
    else
      stop("Argument '", name, "' must be a character vector of length ",
           length, ".",
           call. = FALSE)
  }
  #
  if (length == 1) {
    if (!is.null(nchar) && !(nchar(x) %in% nchar))
      if (length(nchar) == 1 && nchar == 1)
        stop("Argument '", name, "' must be a single character.",
             call. = FALSE)
    else
      stop("Argument '", name, "' must be a character string of length ",
           if (length(nchar) == 2)
             paste0(nchar, collapse = " or ")
           else
             paste0(nchar, collapse = ", "),
           ".",
           call. = FALSE)
  }
  #
  if (!is.character(x) & !is.numeric(x))
    stop("Argument '", name, "' must be a character vector.")
  else {
    if (!is.null(nchar) & any(!(nchar(x) %in% nchar)))
      if (length(nchar) == 1 && nchar == 1)
        stop("Argument '", name, "' must be a vector of single characters.",
             call. = FALSE)
    else
      stop("Argument '", name, "' must be a character vector where ",
           "each element has ",
           if (length(nchar) == 2)
             paste0(nchar, collapse = " or ")
           else
             paste0(nchar, collapse = ", "),
           " characters.",
           call. = FALSE)
  }
  #
  invisible(NULL)
}

chklevel <- function(x, length = 0, ci = TRUE, name = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  #
  # Check for levels of confidence interval / contour level
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  if (ci)
    "level for confidence interval (range: 0-1)"
  else
    "contour levels (range: 0-1)"
  #
  if (!is.numeric(x))
    if (length && length(x) != length)
      stop("Argument '", name, "' must be a numeric of length ", length, ".",
           call. = FALSE)
  else
    stop("Argument '", name, "' must be numeric.",
         call. = FALSE)
  #
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
  #
  if (any(x <= 0, na.rm = TRUE) | any(x >= 1, na.rm = TRUE))
    stop("Argument '", name, "' must be a numeric between 0 and 1.",
         call. = FALSE)
  #
  invisible(NULL)
}

chklogical <- function(x, name = NULL, text = "") {
  #
  # Check whether argument is logical
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (is.numeric(x))
    x <- as.logical(x)
  #
  if (length(x) !=  1 || !is.logical(x) || is.na(x))
    stop("Argument '", name, "' must be a logical",
         if (text != "") " ", text, ".", call. = FALSE)
  #
  invisible(NULL)
}

chknull <- function(x, name = NULL) {
  #
  # Check whether argument is NULL
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (is.null(x))
    stop("Argument '", name, "' is NULL.", call. = FALSE)
  #
  invisible(NULL)
}

chknumeric <- function(x, min, max, zero = FALSE, length = 0,
                       name = NULL, single = FALSE, integer = FALSE,
                       NA.ok = TRUE) {
  if (!missing(single) && single)
    length <- 1
  #
  # Check numeric variable
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (NA.ok)
    x <- x[!is.na(x)]
  else if (anyNA(x))
    stop("Missing values not allowed in argument '", name, "'.",
         call. = FALSE)
  #
  if (length(x) == 0)
    return(NULL)
  #
  if (!is.numeric(x))
    stop("Non-numeric value for argument '", name, "'.",
         call. = FALSE)
  #
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
  #
  if (!missing(min) & missing(max)) {
    if (zero & min == 0 & any(x <= min, na.rm = TRUE))
      stop("Argument '", name, "' must be positive.",
           call. = FALSE)
    else if (any(x < min, na.rm = TRUE))
      stop("Argument '", name, "' must be larger equal ",
           min, ".", call. = FALSE)
  }
  #
  if (missing(min) & !missing(max)) {
    if (zero & max == 0 & any(x >= max, na.rm = TRUE))
      stop("Argument '", name, "' must be negative.",
           call. = FALSE)
    else if (any(x > max, na.rm = TRUE))
      stop("Argument '", name, "' must be smaller equal ",
           min, ".", call. = FALSE)
  }
  #
  if ((!missing(min) & !missing(max)) &&
      (any(x < min, na.rm = TRUE) | any(x > max, na.rm = TRUE)))
    stop("Argument '", name, "' must be between ",
         min, " and ", max, ".", call. = FALSE)
  #
  if (integer && any(!is_wholenumber(x))) {
    if (length(x) == 1)
      stop("Argument '", name, "' must be an integer.",
           call. = FALSE)
    else
      stop("Argument '", name, "' may only contain integers.",
           call. = FALSE)
  }
  #
  invisible(NULL)
}

is_wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
  if (is.numeric(x))
    res <- abs(x - round(x)) < tol
  else
    res <- NA
  #
  res
}

formatN <- function(x, digits = 2, text.NA = "--", big.mark = "",
                    format.whole.numbers = TRUE,
                    monospaced = FALSE) {
  
  outdec <- options()$OutDec
  
  if (!monospaced) {
    if (format.whole.numbers) {
      res <- format(ifelse(is.na(x),
                           text.NA,
                           formatC(x, decimal.mark = outdec,
                                   format = "f", digits = digits,
                                   big.mark = big.mark)
      )
      )
    }
    else {
      res <- format(ifelse(is.na(x),
                           text.NA,
                           ifelse(is_wholenumber(x),
                                  x,
                                  formatC(x, decimal.mark = outdec,
                                          format = "f", digits = digits,
                                          big.mark = big.mark)
                           )
      )
      )
    }
  }
  else {
    x <- round(x, digits)
    res <- ifelse(is.na(x),
                  text.NA,
                  format(x, decimal.mark = outdec, big.mark = big.mark))
  }
  #
  res <- rmSpace(res, end = TRUE)
  #
  res
}

rmSpace <- function(x, end = FALSE, pat = " ") {
  
  if (!end) {
    while (any(substring(x, 1, 1) == pat, na.rm = TRUE)) {
      sel <- substring(x, 1, 1) == pat
      x[sel] <- substring(x[sel], 2)
    }
  }
  else {
    last <- nchar(x)
    
    while (any(substring(x, last, last) == pat, na.rm = TRUE)) {
      sel <- substring(x, last, last) == pat
      x[sel] <- substring(x[sel], 1, last[sel] - 1)
      last <- nchar(x)
    }
  }
  
  x
}

formatCI <- function(lower, upper,
                     bracket.left = gs("CIbracket"),
                     separator = gs("CIseparator"),
                     bracket.right,
                     justify.lower = "right",
                     justify.upper = justify.lower,
                     lower.blank = gs("CIlower.blank"),
                     upper.blank = gs("CIupper.blank"),
                     ...) {
  
  # Change layout of CIs
  #
  chkchar(bracket.left, length = 1)
  chkchar(separator, length = 1)
  if (!missing(bracket.right))
    chkchar(bracket.right, length = 1)
  #
  if (missing(bracket.left)) {
    bracktype <- setchar(bracket.left, c("[", "(", "{", ""))
    #
    if (bracktype == "[") {
      bracketLeft <- "["
      bracketRight <- "]"
    }
    else if (bracktype == "(") {
      bracketLeft <- "("
      bracketRight <- ")"
    }
    else if (bracktype == "{") {
      bracketLeft <- "{"
      bracketRight <- "}"
    }
    else if (bracktype == "") {
      bracketLeft <- ""
      bracketRight <- ""
    }
    #
    bracket.left <- bracketLeft
  }
  #
  if (missing(bracket.right))
    bracket.right <- bracketRight
  
  format.lower <- format(lower, justify = justify.lower)
  format.upper <- format(upper, justify = justify.upper)
  #
  if (!lower.blank)
    format.lower <- rmSpace(format.lower)
  if (!upper.blank)
    format.upper <- rmSpace(format.upper)
  #
  if (separator == "-")
    format.upper <-
    paste0(ifelse(substring(format.upper, 1, 1) == "-", " ", ""),
           format.upper)
  #
  res <- ifelse(lower != "NA" & upper != "NA",
                paste0(bracket.left,
                       format.lower,
                       separator,
                       format.upper,
                       bracket.right),
                "")
  #
  res
}

#' A heatplot to visualize the ranking in a multivariate network meta-analysis
#' 
#' @description
#' This function produces a heatplot displaying the results of function
#' \code{\link{mvrank}}. The graph can be used to visualize the ranking output
#' when the method used to rank the treatments is either the surface under the
#' cumulative ranking curve (SUCRA) or the probability of best value (pBV).
#' 
#' @param x An object of class \code{\link{mvrank}}.
#' @param sort An optional argument to define an outcome to be used as a
#'   reference when sorting the order of treatments on the x-axis.
#'   If \code{NULL}, then the first outcome, as it appears in
#'   \code{\link{mvrank}}, is used as the reference. The user can either
#'   provide a single number corresponding to the position of the outcome in
#'   the \code{\link{mvrank}} object or a character string with the name of
#'   the outcome; can be abbreviated.
#' @param col.num The color of the numbers in the squares (default: "white").
#' @param num.size The size of the numbers in the squares (default: 4.5).
#' @param col.low The color for the low end of the gradient
#'   (default: "lightblue").
#' @param col.high The color for the high end of the gradient
#'   (default: "darkblue").
#' @param width.bar The width of the bars in the graph (default: 1.2).
#' @param angle.x The angle (in [0, 360]) to rotate the text appearing on
#'   the x-axis (default: 0).
#' @param angle.y The angle (in [0, 360]) to rotate the text appearing on the
#'   y-axis (default: 0).
#' @param hjust.x Horizontal justification for the text on the x-axis
#'   (default: 0.5).
#' @param hjust.y Horizontal justification for the text on the y-axis
#'   (default: 1).
#' @param hjust.legend Horizontal justification for the text in the legend
#'   title (default: 0.5).
#' @param size.x Text size (in pts) on the x-axis (default: 8).
#' @param size.y Text size (in pts) on the y-axis (default: 10).
#' @param legend.position Position of the legend. Options are "top", "bottom",
#'   "right" (default), and "left"; can be abbreviated.
#' @param legend.direction Direction of the legend. Options are "vertical"
#'   (default) and "horizontal"; can be abbreviated.
#' @param \dots Additional arguments (ignored).
#'
#' @return
#' A \code{ggplot} object.
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Rank the treatments
#' ranks_sucra <- mvrank(mvnma_all, 
#'   small.values = c("undes", "undes", "des", "des", "des"),
#'   method = "SUCRA")
#' ranks_sucra
#' 
#' # Create a heatplot sorting the results according to the first outcome
#' # (default), i.e., Early_Response
#' heatplot(ranks_sucra)
#' 
#' # Create a heatplot sorting the results according to the second outcome
#' # appearing in ranks_sucra (i.e Early_Remission)
#' heatplot(ranks_sucra, sort = 2)
#' 
#' # Create a heatplot sorting the results by explicitly mentioning the
#' # name of the outcome Early_Remission
#' heatplot(ranks_sucra, sort = "Early_Remission")
#' 
#' @method heatplot mvrank 
#' @export

heatplot.mvrank <- function(x,
                            sort = NULL,
                            col.num = "white",
                            num.size = 4.5,
                            col.low = "lightblue",
                            col.high = "darkblue",
                            width.bar = 1.2,
                            angle.x = 0,
                            angle.y = 0, 
                            hjust.x = 0.5,
                            hjust.y = 1,
                            hjust.legend = 0.5,
                            size.x = 8,
                            size.y = 10,
                            legend.position = "right",
                            legend.direction = "vertical",
                            ...) {
  
  chkclass(x, "mvrank")
  #
  method <- attr(x, "method")
  #
  if (!(method %in% c("SUCRA", "pBV")))
    stop("Heatplot can only be produced for mvnma object created with ",
         "'method = \"SUCRA\"' or \"pBV\".",
         call. = FALSE)
  
  outcome <- treatment <- val <- NULL
  #
  legend.position <-
    setchar(legend.position, c("top", "bottom", "right", "left"))
  #
  legend.direction <- setchar(legend.direction, c("horizontal", "vertical"))
  
  # Create a data frame with all ranks
  #
  dat.ranks <- vector("list")
  
  for (i in 1:length(x)) {
    dat.ranks[[i]] <- x[[i]] 
    dat.ranks[[i]]$outcome <- names(x)[i]
  }
  #
  dat.ranks <- bind_rows(dat.ranks)
  
  # Make sure that argument sort is either a character (can be abbreviated) or
  # a numeric value
  #
  if (is.null(sort)) {
    sort <-  names(x)[1]
    order <- dat.ranks %>% filter(outcome==sort)
  }
  else {
    if (length(sort) > 1) {
      stop("Argument 'sort' must be of length 1.", call. = FALSE)
    }
    #
    if ((!is.numeric(sort)) & (!is.character(sort))) {
      stop("Argument 'sort' must be either a character string or a ",
           "numeric value of length 1.", call. = FALSE)
    }
    else if (is.character(sort)) {
      sort <- setchar(sort, names(x))
      order <- dat.ranks %>% filter(outcome == sort)
    }
    else if (is.numeric(sort)) {
      if (sort > length(x)) {
        stop("Argument 'sort' must be a number between ", 1," and ", length(x),
             ".", call. = FALSE)
      }
      #
      sort <- names(x)[sort] 
      order <- dat.ranks %>% filter(outcome==sort)
    }
  }
  
  # Create order for treatments and outcomes
  #
  order <- order$treatment
  #
  dat.ranks$treatment <- factor(dat.ranks$treatment, levels = order)
  #
  dat.ranks$outcome <- factor(dat.ranks$outcome, levels = names(x))
  #
  dat.ranks$outcome <- relevel(dat.ranks$outcome, ref = sort)
  #
  names(dat.ranks) <- c("treatment","val","outcome")
  
  # Create the heatplot
  #
  p <- ggplot(dat.ranks, aes(treatment, fct_rev(outcome), fill = val)) + 
    geom_tile(color = "black") +
    geom_text(aes(label = paste(format(round(val,digits = 2), nsmall = 2))),
              size = num.size, color = col.num) +
    scale_fill_gradient(low = col.low, high = col.high) + 
    guides(fill = guide_colourbar(label = FALSE, ticks = FALSE,
                                  barwidth = width.bar))+
    xlab("") +
    ylab("") +
    labs(fill = method) +
    scale_y_discrete(expand = c(0, 0)) +
    theme_void() +
    theme(
      # bold font for legend text
      legend.text = element_text(face = "bold"),
      # set thickness of axis ticks
      axis.ticks.y = element_blank(),
      axis.text = element_text(face = "bold"),
      #remove plot background
      plot.background = element_blank(),
      #remove plot border
      panel.border = element_blank(),
      axis.text.x = element_text(angle = angle.x, hjust = hjust.x,
                                 size = size.x),
      axis.text.y = element_text(angle = angle.y, hjust = hjust.y,
                                 size = size.y),
      legend.position = legend.position,
      legend.direction = legend.direction,
      legend.title = element_text(hjust = hjust.legend,)
    )
  #
  attr(p, "dat.ranks") <- dat.ranks
  attr(p, "method") <- method
  #
  p
}

#' Rank treatments across all outcomes using the spie chart method 
#' 
#' @description
#' This function employs the spie chart approach to combine
#' outcome-specific treatment hierarchies obtained in terms of the
#' probabilistic ranking metrics:
#' \itemize{
#' \item Surface Under the Cumulative Ranking curve (SUCRA), or
#' \item probability of best value (pBV).
#' }
#' 
#' @param x An object of class \code{\link{mvrank}}.
#' @param weights Outcome weights. The weights should always sum to 1. If not
#'   then they are standardized. If NULL, the function will assume equal outcome
#'   weights. 
#' @param \dots Additional arguments passed to \code{\link[ggplot2]{ggplot}}
#'   function.
#' 
#' @return
#' A data frame with the area under the spie chart as amalgamated treatment
#' hierarchy across outcomes.
#' 
#' @references 
#' Daly CH, Mbuagbaw L, Thabane L, Straus SE, Hamid JS (2020):
#' Spie charts for quantifying treatment effectiveness and safety in multiple
#' outcome network meta-analysis: a proof-of-concept study.
#' \emph{BMC Med Res Methodol}, \bold{20}, 266
#' 
#' @details
#' The spie chart is a modified pie chart in which each sector corresponds to
#' a treatment and its radius is proportional to a ranking metric, so that
#' sector area encodes treatment performance (Daly et al., 2020). This function
#' constructs one spie chart per outcome using SUCRA or probability of best
#' values (pBV) metrics, where each metric determines the radius of the
#' corresponding sector.
#' 
#' An amalgamated treatment hierarchy across outcomes is then derived by
#' averaging the sector areas across outcomes, providing a quantitative
#' summary of overall treatment performance.
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Rank treatments using SUCRAs
#' ranks12 <- mvrank(mvnma12, method = "sucra",
#'   small.values = c("undes", "undes"))
#' ranks12
#' 
#' # Get amalgamated treatment hierarchy
#' spiechart(ranks12)
#' 
#' @rdname spiechart
#' @method spiechart mvrank
#' @export

spiechart.mvrank <- function(x, weights = NULL, ...) {
  
  chkclass(x, "mvrank")
  #
  trts <- sort(attr(x, "common_trts"))
  outcomes <- names(x)
  
  if (attr(x, "method") %in% c("SUCRA", "pBV")) {
    # Get rid of warning "no visible binding for global variable"
    treatment <- NULL
    
    s <- vector("list")
    #
    x.common <- attr(x,"ranks.common")
    #
    for (i in seq_len(length(x))) {
      dat.i <- x.common[[i]] %>% 
        filter(treatment %in% trts) %>% 
        arrange(treatment)
      #
      s[[i]] <- as.data.frame(dat.i[, 2])
      #
      names(s[[i]]) <- paste("ranks", i, sep = "_")
    }
    #
    rankings <- bind_cols(s)
    names(rankings) <- outcomes
    row.names(rankings) <- dat.i$treatment
    #
    res <- spiechart_internal(rankings, weights)
  }
  else
    stop("Spie chart area can only be calculated for method SUCRA and pBV")
  
  class(res) <- c("spiechart", class(res))
  attr(res, "weights") <- weights
  #
  res
}


#' @rdname spiechart
#' @export spiechart

spiechart <- function(x, ...)
  UseMethod("spiechart")


#' @method print spiechart
#' @export

print.spiechart <- function(x, digits = 2, ...) {
  chkclass(x, "spiechart")
  #
  # Get rid of warning 'Undefined global functions or variables'
  #
  area <- NULL
  #
  x %<>%
    rename(Area = area) %>%
    round(digits = digits)
  #
  prmatrix(x, quote = FALSE, right = TRUE)
  #
  invisible(NULL)
}

#' Outcome-specific treatment rankings in multivariate network meta-analysis.
#' 
#' @description
#' Produces outcome-specific treatment rankings in multivariate
#' network meta-analysis based on the output of the \code{\link{mvnma}}
#' function.
#' 
#' @param x An object of class \code{\link{mvnma}}.
#' @param small.values A character vector specifying for each outcome whether
#'   small treatment effects indicate a beneficial ("desirable") or harmful
#'   ("undesirable") effect, can be abbreviated.
#' @param method The ranking method to be used. Three methods are currently
#'   supported. The SUCRA method (specified as \code{method = "SUCRA"}) is the
#'   default approach. The probability of best value method (specified as
#'   \code{method = "pBV"}) and the mean and median ranks (specified as
#'   \code{method = "ranks"}) are also supported.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param \dots Additional arguments (ignored)
#' 
#' @description
#' This function produces outcome-specific treatment rankings in multivariate
#' network meta-analysis based on the output of the \code{\link{mvnma}}
#' function. Two ranking methods (argument \code{method}) are currently
#' supported (Salanti et al., 2011):
#' \itemize{
#' \item Surface under the cumulative ranking curve (SUCRA) method,
#' \item Probability of best value (pBV) method.
#' }
#'  
#' @return
#' The function returns an 'mvrank' object which is a list consisting of
#' a data frame with the variables 'treatment' and either 'SUCRA' or 'pBV'
#' for each outcome in the multivariate network meta-analysis.
#' 
#' @references
#' Salanti G, Ades AE, Ioannidis JP (2011):
#' Graphical methods and numerical summaries for presenting results
#' from multiple-treatment meta-analysis: an overview and tutorial.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{64}, 163--71
#' 
#' @examples
#' # Locate file "mvnma_examples.rda" with mvnma() results
#' .fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
#' load(.fname)
#' 
#' # Rank treatments using SUCRAs (default)
#' ranks_sucra <- mvrank(mvnma_all, 
#'   small.values = c("undes", "undes", "des", "des", "des"))
#' #
#' ranks_sucra
#' 
#' \donttest{
#' # Rank treatments using pBV
#' ranks_pBV <- mvrank(mvnma_all,
#'   small.values = c("undes", "undes", "des", "des", "des"),
#'   method = "pBV")
#' #
#' ranks_pBV         
#' 
#' # Rank treatments using mean and median ranks
#' ranks_mean_median <- mvrank(mvnma_all,
#'   small.values = c("undes", "undes", "des", "des", "des"),
#'   method = "ranks")
#' #
#' ranks_mean_median
#' }
#' 
#' @export mvrank                

mvrank <- function(x, small.values, method = "SUCRA") {
  
  chkclass(x, "mvnma")
  #
  small.values <- setchar(small.values, c("undesirable", "desirable"))
  method <- setchar(method, c("SUCRA", "pBV", "ranks"))
  chkchar(method, length = 1)
  method.model <- attr(x, "method.model")
  n.domain <- attr(x,"n.domain")
  #
  x <- x[names(x) != "cor"]
  #
  if (method.model == "DM") {
    if (is.null(n.domain)) {
      x <- x[names(x) != "sigma"]
    }
    else{
      x <- x[!(names(x) %in% c("sigma1", "sigma2"))]
    }
  }
  #
  outcomes <- attr(x, "names")
  #
  colname_list <- lapply(x, function(k) colnames(k$samples))
  common_trts <- Reduce(intersect, colname_list)
  
  # Get rid of warning "no visible binding for global variable"
  #
  treatment <- pBV <- SUCRA <- Freq <- median_rank <- mean_rank <-
    lower.CrI <- upper.CrI <- NULL
  
  # Extract samples and create rankograms for each outcome
  #
  n.out <- length(outcomes)
  #
  d <- d_common <- n.trts <- trts <- rank_out <- rank_out_common <-
    ranks <- ranks.common <- quant <- quant_common <-
    rnk <- rnk_common <- vector("list", n.out)
  #
  for (i in seq_len(n.out)) {
    d[[i]] <- x[[i]]$samples
    n.trts[[i]] <- ncol(d[[i]])
    trts[[i]] <- colnames(d[[i]])
    if(length(setdiff(trts[[i]],common_trts))==0){
      d_common[[i]] <- d[[i]]  
    }
    else{
      d_common[[i]] <- d[[i]] %>% select(common_trts)
    }
    
    rank_out[[i]] <- rankogram(d[[i]], small.values = small.values[i])
    rank_out_common[[i]] <- rankogram(d_common[[i]], small.values = small.values[i])
    #
    if (method == "pBV") {
      ranks.i <- data.frame(pBV = rank_out[[i]]$ranking.matrix.random[, 1])
      #
      ranks.i$treatment <- row.names(ranks.i)
      row.names(ranks.i) <- NULL
      #
      ranks.i %<>% select(treatment, pBV) %>% arrange(desc(pBV))
      
      # recalculate only for the common treatments
      ranks.i.common <- data.frame(pBV = rank_out_common[[i]]$ranking.matrix.random[, 1])
      #
      ranks.i.common$treatment <- row.names(ranks.i.common)
      row.names(ranks.i.common) <- NULL
      #
      ranks.i.common %<>% select(treatment, pBV) %>% arrange(desc(pBV))
    }
    else if (method == "SUCRA") {
      ranks.i <- rank_out[[i]]$ranking.random
      ranks.i <- cbind.data.frame(names(ranks.i), unname(ranks.i))
      names(ranks.i) <- c("treatment", "SUCRA")
      ranks.i %<>% arrange(desc(SUCRA))
      # Recalculate for the common treatments
      ranks.i.common <- rank_out_common[[i]]$ranking.random
      ranks.i.common <-
        cbind.data.frame(names(ranks.i.common), unname(ranks.i.common))
      names(ranks.i.common) <- c("treatment", "SUCRA")
      ranks.i.common %<>% arrange(desc(SUCRA))
    }
    else if (method == "ranks") {
      if (small.values[i] == "undesirable")
        rnk[[i]] <- apply(-d[[i]], 1, rank, ties.method = "random")
      else
        rnk[[i]] <- apply(d[[i]], 1, rank, ties.method = "random") 
      
      quant[[i]] <-
        as.data.frame(
          t(apply(rnk[[i]], 1,
                  function(row) {
                    quantile(row, probs = c(0.025, 0.5, 0.975), na.rm = TRUE)
                    }
                  )
            )
          )
      #
      quant[[i]]$treatment <- row.names(quant[[i]])
      quant[[i]]$mean_ranks <- rowMeans(rnk[[i]])
      #
      row.names(quant[[i]]) <- NULL
      names(quant[[i]]) <-
        c("lower.CrI", "median_rank", "upper.CrI", "treatment", "mean_rank")
      
      ranks.i <- quant[[i]] %<>%
        select(treatment, median_rank, mean_rank, lower.CrI, upper.CrI) %>%
        arrange(mean_rank)
      
      # Recalculate for the common treatments
      #
      if (small.values[i] == "undesirable")
        rnk_common[[i]] <-
        apply(-d_common[[i]], 1, rank, ties.method = "random")
      else
        rnk_common[[i]] <-
        apply(d_common[[i]], 1, rank, ties.method = "random") 
      
      quant_common[[i]] <-
        as.data.frame(
          t(apply(rnk_common[[i]], 1,
                  function(row) {
                    quantile(row, probs = c(0.025, 0.5, 0.975), na.rm = TRUE)
                    }
                  )
            )
          )
      #
      quant_common[[i]]$treatment <- row.names(quant_common[[i]])
      quant_common[[i]]$mean_ranks <- rowMeans(rnk_common[[i]])
      #
      row.names(quant_common[[i]]) <- NULL
      names(quant_common[[i]]) <-
        c("lower.CrI", "median_rank", "upper.CrI", "treatment", "mean_rank")
      
      ranks.i.common <- quant_common[[i]] %<>%
        select(treatment, median_rank, mean_rank, lower.CrI, upper.CrI) %>%
        arrange(mean_rank)
    }
    #
    ranks[[i]] <- ranks.i
    ranks.common[[i]] <- ranks.i.common
  }
  #
  names(ranks) <- outcomes
  class(ranks) <- "mvrank"
  #
  names(ranks.common) <- outcomes
  class(ranks.common) <- "mvrank"
  #
  # common_trts <- as.data.frame(table(unlist(trts))) %>% filter(Freq == n.out)
  # common_trts <- common_trts$Var1
  #
  attr(ranks, "common_trts") <- common_trts
  attr(ranks, "method") <- method
  attr(ranks, "ranks.common.trts") <- ranks.common
  #
  ranks
}


#' @rdname mvrank 
#' @method print mvrank
#' @export

print.mvrank <- function(x, digits = gs("digits"), ...) {
  
  chkclass(x, "mvrank")
  #
  chknumeric(digits, min = 0, length = 1)
  #
  nam <- names(x)
  
  # Get rid of warning "no visible binding for global variable"
  treatment <- NULL
  #
  for (i in seq_along(nam)) {
    cat(paste0(if (i > 1) "\n" else "", "Outcome: ", nam[i], "\n\n"))
    #
    dat.i <- x[[i]]
    rownames(dat.i) <- dat.i$treatment
    dat.i %<>% select(-treatment)
    #
    nam.i <- names(dat.i)
    #
    for (j in nam.i)
      dat.i[[j]] <- formatN(dat.i[[j]], digits = digits)
    #
    prmatrix(dat.i, quote = FALSE, right = TRUE)
  }
  #
  invisible(NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linechart.R
\name{linechart}
\alias{linechart}
\title{Line chart showing the VIKOR results}
\usage{
linechart(
  x,
  sort = "Q",
  exclude = "none",
  n = nrow(x),
  linewidth = 1.1,
  size = 2,
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{vikor}}.}

\item{sort}{A character specifying the order of treatments on the x-axis.
By default, the order is according to the Q-metric (\code{"Q"}). Ordering
treatments according to the S (\code{"S"}) and R (\code{"R"}) metrics is
also possible.}

\item{exclude}{A character specifying a metric that will not be displayed in
the graph. By default, all metrics are displayed (\code{"none"}).
Alternative options are to exclude either the Q-metric (\code{"Q"}),
the S-metric (\code{"S"}), or the R-metric (\code{"R"}).}

\item{n}{A numeric value indicating the number of treatments to be plotted
in the graph. By default, all treatments are displayed. If specified,
only the first \code{"n"} treatments according to the hierarchy specified
from \code{"sort"} argument are plotted.}

\item{linewidth}{A numeric value specifying the width of the lines
(default: 1.1).}

\item{size}{A numeric value specifying the size of the points
(default: 2).}

\item{\dots}{Additional arguments passed to \code{\link[ggplot2]{ggplot}}
function.}
}
\value{
A \code{ggplot} object.
}
\description{
A line chart showing the results of VišeKriterijumska Optimizacija I
Kompromisno Rešenje (VIKOR) across the three metrics Q, S and R for each
treatment. Within each metric, lower values represent better treatment
performance.
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Rank treatments using SUCRAs (default)
ranks12 <- mvrank(mvnma12, small.values = c("undes", "undes"))
ranks12

# Get the best compromise solution across the efficacy outcomes
vk12 <- vikor(ranks12)

# Visualize the results with default settings
linechart(vk12)

# Sort by the "R" metric
linechart(vk12, sort = "R")

# Sort by the "R" metric and include only the first 3 treatments
linechart(vk12, sort = "R", n = 3)

# Exclude the "R" metric
#
linechart(vk12, exclude = "R")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hasse.R
\name{hasse.mvrank}
\alias{hasse.mvrank}
\title{Hasse diagram}
\usage{
\method{hasse}{mvrank}(x, ...)
}
\arguments{
\item{x}{An object of class \code{\link{mvrank}}.}

\item{\dots}{Additional arguments passed on to
\code{\link[netmeta]{hasse.netposet}}.}
}
\value{
A Hasse diagram is shown in the active graphics window.
}
\description{
This function generates a Hasse diagram for a partial order of treatment
ranks in a multivariate network meta-analysis.
}
\details{
Generate a Hasse diagram (Carlsen & Bruggemann, 2014) for a partial order of
treatment ranks in a network meta-analysis (Rücker & Schwarzer, 2017).

This R function is a wrapper function for
\code{\link[netmeta]{hasse.netposet}}.
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Print odds ratios for efficacy outcomes
outc <- names(mvnma12)[names(mvnma12) != "cor"]
#
for (i in outc) {
  cat(paste0("\nOutcome: ", i, "\n\n"))
  print(round(exp(mvnma12[[i]]$TE.random), 2))
}

# Rank treatments using SUCRAs
ranks12 <- mvrank(mvnma12, method = "sucra",
  small.values = c("undes", "undes"))
ranks12

# Get the Hasse diagram for the efficacy outcomes
hasse(ranks12)

}
\references{
Carlsen L, Bruggemann R (2014):
Partial order methodology: a valuable tool in chemometrics.
\emph{Journal of Chemometrics},
\bold{28}, 226--34

Rücker G, Schwarzer G (2017):
Resolve conflicting rankings of outcomes in network meta-analysis:
Partial ordering of treatments.
\emph{Research Synthesis Methods},
\bold{8}, 526--36
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvrank.R
\name{mvrank}
\alias{mvrank}
\alias{print.mvrank}
\title{Outcome-specific treatment rankings in multivariate network meta-analysis.}
\usage{
mvrank(x, small.values, method = "SUCRA")

\method{print}{mvrank}(x, digits = gs("digits"), ...)
}
\arguments{
\item{x}{An object of class \code{\link{mvnma}}.}

\item{small.values}{A character vector specifying for each outcome whether
small treatment effects indicate a beneficial ("desirable") or harmful
("undesirable") effect, can be abbreviated.}

\item{method}{The ranking method to be used. Three methods are currently
supported. The SUCRA method (specified as \code{method = "SUCRA"}) is the
default approach. The probability of best value method (specified as
\code{method = "pBV"}) and the mean and median ranks (specified as
\code{method = "ranks"}) are also supported.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{\dots}{Additional arguments (ignored)}
}
\value{
The function returns an 'mvrank' object which is a list consisting of
a data frame with the variables 'treatment' and either 'SUCRA' or 'pBV'
for each outcome in the multivariate network meta-analysis.
}
\description{
Produces outcome-specific treatment rankings in multivariate
network meta-analysis based on the output of the \code{\link{mvnma}}
function.

This function produces outcome-specific treatment rankings in multivariate
network meta-analysis based on the output of the \code{\link{mvnma}}
function. Two ranking methods (argument \code{method}) are currently
supported (Salanti et al., 2011):
\itemize{
\item Surface under the cumulative ranking curve (SUCRA) method,
\item Probability of best value (pBV) method.
}
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Rank treatments using SUCRAs (default)
ranks_sucra <- mvrank(mvnma_all, 
  small.values = c("undes", "undes", "des", "des", "des"))
#
ranks_sucra

\donttest{
# Rank treatments using pBV
ranks_pBV <- mvrank(mvnma_all,
  small.values = c("undes", "undes", "des", "des", "des"),
  method = "pBV")
#
ranks_pBV         

# Rank treatments using mean and median ranks
ranks_mean_median <- mvrank(mvnma_all,
  small.values = c("undes", "undes", "des", "des", "des"),
  method = "ranks")
#
ranks_mean_median
}

}
\references{
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.mvrank.R
\name{plot.mvrank}
\alias{plot.mvrank}
\title{Scatter plot to visualize the ranking of two outcomes in a multivariate network meta-analysis}
\usage{
\method{plot}{mvrank}(
  x,
  which = 1:2,
  pos = 1,
  cex.point = 1,
  cex.label = 0.7,
  pch = 19,
  xlim = c(0, 1),
  ylim = c(0, 1),
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{mvrank}}.}

\item{which}{A mandatory numeric vector of length 2 specifying which
outcomes should be plotted. For example, setting "outcome = c(2, 3)"
implies that a scatter plot will be generated plotting the rankings
of outcomes 2 and 3.}

\item{pos}{Position of treatment labels.}

\item{cex.point}{a numeric value specifying the size of the points
(default: 1)}

\item{cex.label}{a numeric value specifying the size of the point labels in
the plot (default: 0.7)}

\item{pch}{a vector of plotting characters or symbols (default: 19)}

\item{xlim}{the x limits of the plot}

\item{ylim}{the y limits of the plot}

\item{\dots}{Additional arguments for \code{\link{plot}} function.}
}
\value{
A scatter plot is shown in the active graphics window.
}
\description{
Draw a scatter plot in the active graphics window.
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Rank treatments using SUCRAs (default)
ranks12 <- mvrank(mvnma12, small.values = c("undes", "undes"))
ranks12

# Visualize SUCRAs in a scatter plot with outcome 1 on the x-axis and
# outcome 2 on the y-axis
plot(ranks12)

# Visualize SUCRAs in a scatter plot with outcome 2 on the x-axis and
# outcome 1 on the y-axis
plot(ranks12, which = 2:1)

}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvnma-package.R
\docType{package}
\name{mvnma-package}
\alias{mvnma-package}
\title{mvnma: Brief overview of methods}
\description{
R package \bold{mvnma} provides R functions for Bayesian multivariate
network meta-analysis (mvNMA). The mvNMA model supported by this package
refers to the single correlation coefficient model, interpreted as an
amalgam of within- and across-outcome correlations (Efthimiou et al., 2015)
which is a generalisation of Riley et al. (2008).
In this way, the model does not depend on the extraction of within-study
outcome correlations, which are seldom reported at the study level.

DuMouchel priors assuming constant relative treatment effects across
outcomes and enabling information sharing can be used (DuMouchel & Harris,
1983). This may improve precision but can introduce bias when outcomes from
different domains (e.g., efficacy and safety) are analyzed jointly.

The treatment effect estimates
and confidence intervals can be summarised both in terms of per-outcome
treatment hierarchies and in terms of an across-outcomes benefit-risk
assessment. The former is possible using ranking methods such as the surface
under the cumulative ranking curve (SUCRA) (Salanti et al., 2011), the
probability of best value, and median (or mean) ranks, each accompanied by a
credible interval.

A benefit-risk assessment is possible through the VišeKriterijumska
Optimizacija I Kompromisno Rešenje (VIKOR) method (Opricovic & Tzeng, 2004;
Opricovic, 2011). This approach, originally proposed in the field of
multi-criteria decision analysis, uses a deterministic algorithm to provide
an amalgamated treatment hierarchy across outcomes and explicitly identify
the set of treatments that offer the best compromise between benefits and
harms across all outcomes.

Since the output of the method relies on Markov Chain Monte Carlo (MCMC)
sampling, convergence can be checked using a series of diagnostics, including
trace plots, density plots, and the R-hat statistic. Finally, this package
offers the option to visualise the results of the mvNMA model through forest
plots, which display the treatment effect estimates; scatter plots, which
show the per-outcome rankings for any pair of outcomes; and Hasse diagrams
(Carlsen & Bruggemann, 2014), which visualise the partial order of treatments
across all outcomes (Rücker & Schwarzer, 2017), as well as alternative
methods to yield a between-outcomes hierarchy, such as the spie chart method
(Daly et al., 2020).
}
\details{
The R package \bold{mvnma} provides the following functions:
\itemize{
\item Function \code{\link{mvnma}} to perform a Bayesian multivariate
  network meta-analysis.
\item Function \code{\link{mvrank}} to get outcome-specific treatment
 rankings.
\item Function \code{\link{vikor}} to rank treatments across all outcomes
  using the VIKOR multi-criteria decision analysis method. Additionally,
  the function evaluates the concrete conditions defined by the VIKOR method
  and identifies the set of treatments that offer the best compromise
  between benefits and harms across all outcomes.
\item Function \code{\link{forest.mvnma}} to visualize the results of the
  mvNMA model in terms of treatment effect estimates.
\item Function \code{\link{plot.mvrank}} to visualize per outcome ranking
  results for any pair of outcomes.
\item Function \code{\link{hasse.mvrank}} to visualize the partial order of
  the treatment across all outcomes.
\item Function \code{\link{heatplot.mvrank}} to visualize in a heatplot
  the results in terms of outcome specific rankings.
\item Function \code{\link{linechart}} to visualize the results of the three
  metrics calculated by the VIKOR method.
\item Function \code{\link{as.mcmc.mvnma}} an auxiliary function to extract
  an MCMC object. This makes any \bold{mvnnma} object compatible with the
  convergence checks performed by the R package \bold{coda}.
 \item Function \code{\link{spiechart}} to calculate a between outcomes
   hierarchy using the spie charts method.
}

Type \code{help(package = "mvnma")} for a listing of R functions
available in \bold{mvnma}.

Type \code{citation("mvnma")} on how to cite \bold{mvnma}
in publications.

To report problems and bugs, please send an email to Theodoros
Evrenoglou <theodoros.evrenoglou@uniklinik-freiburg.de>.

The development version of \bold{mvnma} is available on GitHub
\url{https://github.com/TEvrenoglou/mvnma}.
}
\references{
Carlsen L, Bruggemann R (2014):
Partial order methodology: a valuable tool in chemometrics.
\emph{Journal of Chemometrics},
\bold{28}, 226--34

Daly CH, Mbuagbaw L, Thabane L, Straus SE, Hamid JS (2020):
Spie charts for quantifying treatment effectiveness and safety in multiple
outcome network meta-analysis: a proof-of-concept study.
\emph{BMC Med Res Methodol}, \bold{20}, 266

DuMouchel WH, Harris JE (1983):
Bayes methods for combining the results of cancer studies in humans and
other species.
\emph{Journal of the American Statistical Association},
\bold{78}, 293--308

Efthimiou O, Mavridis D, Riley RD, Cipriani A, Salanti G (2015):
Joint synthesis of multiple correlated outcomes in networks of interventions.
\emph{Biostatistics}, 
\bold{16}, 84--97

Opricovic S, Tzeng GH (2004):
Compromise solution by MCDM methods: A comparative analysis of VIKOR and
TOPSIS.
\emph{European Journal of Operational Research},
\bold{156}, 445--55

Opricovic S (2011):
Fuzzy VIKOR with an application to water resources planning. 
\emph{Expert Systems with Applications}, 
\bold{38}, 12983--90

Riley RD, Thompson JR, Abrams KR (2008):
An alternative model for bivariate random-effects meta-analysis when the
within-study correlations are unknown.
\emph{Biostatistics},
\bold{9}, 172--86

Rücker G, Schwarzer G (2017):
Resolve conflicting rankings of outcomes in network meta-analysis:
Partial ordering of treatments.
\emph{Research Synthesis Methods},
\bold{8}, 526--36
                                            
Salanti G, Ades AE, Ioannidis JP (2011):
Graphical methods and numerical summaries for presenting results
from multiple-treatment meta-analysis: an overview and tutorial.
\emph{Journal of Clinical Epidemiology},
\bold{64}, 163--71
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/TEvrenoglou/mvnma}
}

}
\author{
Theodoros Evrenoglou <theodoros.evrenoglou@uniklinik-freiburg.de>,
  Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/VIKOR.R
\name{vikor.mvrank}
\alias{vikor.mvrank}
\alias{vikor.matrix}
\alias{vikor}
\alias{print.vikor}
\title{Rank treatments across all outcomes using the VišeKriterijumska Optimizacija
I Kompromisno Rešenje (VIKOR) multi-criteria decision analysis method}
\usage{
\method{vikor}{mvrank}(x, weights = NULL, v = 0.5, ...)

\method{vikor}{matrix}(x, weights = NULL, v = 0.5, ...)

vikor(x, ...)

\method{print}{vikor}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{\link{mvrank}} or a matrix.}

\item{weights}{Outcome weights. The weights should always sum to 1. If not
then they are standardized. If NULL, the function will assume equal outcome
weights.}

\item{v}{A scalar from 0 to 1 interpreted as the weight of the decision
making process. Following guidance from the multi-criteria decision
analysis field it is set to 0.5.}

\item{\dots}{Additional arguments (ignored).}

\item{digits}{A numeric specifying the number of digits to print the
ranking matrix Q.}
}
\value{
The function returns a 'vikor' object. This consists of three ranking lists
which are the following:
\itemize{
\item A ranking list Q referring to the ranking when balancing both each
  treatment's overall and worst performance. This is the main ranking list
  of the method. 
\item A ranking list S referring to the ranking in terms of each treatment's
  overall performance.
\item A ranking list R referring to the ranking in terms of penalising each
  treatment's worst performance.
}
In addition to the ranking lists, the function also evaluates the necessary
conditions defined by the VIKOR method and returns a message indicating the
set of compromise solutions.
}
\description{
This function employs the VišeKriterijumska Optimizacija I Kompromisno
Rešenje (VIKOR) method to analyze all outcome-specific ranking lists.
It provides both an amalgamated ranking list and guidance on
which treatments correspond to the best compromise solutions.
}
\details{
This function takes a single mandatory argument, which is either an object
of class \code{\link{mvrank}} or a matrix. It then uses the multi-criteria
decision analysis method VišeKriterijumska Optimizacija I Kompromisno
Rešenje (VIKOR) to produce an amalgamated ranking list across
all outcomes (Opricovic & Tzeng, 2004).

The standard VIKOR approach is applied when the \code{method} argument is
set to \code{"sucra"} or \code{"pBV"} in \code{\link{mvrank}}.

A fuzzy VIKOR method (Opricovic, 2011) is applied when outcome-specific
rankings are expressed in terms of median ranks and 95\% credible intervals.
The latter is possible when the \code{\link{mvrank}} object is created with
\code{method = "ranks"}.

In both cases, the final ranking list is calculated based on treatments
common across all outcomes. Treatments not present across all outcomes are
excluded internally.

Using the argument 'weights' the users can specify the weight that each
outcome should have in the decision making process. For each outcome this
argument should have a value from 0 to 1 while the sum of all outcome
weights should be 1. If the sum of all weights is not 1, then these are
internally standardize to achieve this. The standardized weight values
are returned as a message to the user. Finally, if NULL then equal weights
are assumed across all outcomes.

The argument 'v' specifies the weight of the decision making process.
The VIKOR method is a compromise programming approach that aims to balance
between each treatments overall and worst performance across all outcomes.
The balance between these two criteria is achieved using the parameter 'v'
which takes values from 0 to 1. Values close to 1 will give more weight to
the treatment's overall performance while values close to 0 will give more
weight to penalize the treatment's worst performance. The most common
choice of 'v' is typically 0.5 (default also here), thereby allowing for a
balanced decision making between treatment's overall and worst performance.
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Print the results of a bivariate network meta-analysis
mvnma12

# Rank treatments using SUCRAs
ranks12 <- mvrank(mvnma12, method = "sucra",
  small.values = c("undes", "undes"))
ranks12

# Get the best compromise solution across the efficacy outcomes
vikor(ranks12)

# Use larger weight for response than remission
vikor(ranks12, weights = c(0.6, 0.3))

}
\references{
Opricovic S, Tzeng GH (2004):
Compromise solution by MCDM methods: A comparative analysis of VIKOR and
TOPSIS.
\emph{European Journal of Operational Research},
\bold{156}, 445--55

Opricovic S (2011):
Fuzzy VIKOR with an application to water resources planning. 
\emph{Expert Systems with Applications}, 
\bold{38}, 12983--90
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spiechart.R
\name{spiechart.mvrank}
\alias{spiechart.mvrank}
\alias{spiechart}
\title{Rank treatments across all outcomes using the spie chart method}
\usage{
\method{spiechart}{mvrank}(x, weights = NULL, ...)

spiechart(x, ...)
}
\arguments{
\item{x}{An object of class \code{\link{mvrank}}.}

\item{weights}{Outcome weights. The weights should always sum to 1. If not
then they are standardized. If NULL, the function will assume equal outcome
weights.}

\item{\dots}{Additional arguments passed to \code{\link[ggplot2]{ggplot}}
function.}
}
\value{
A data frame with the area under the spie chart as amalgamated treatment
hierarchy across outcomes.
}
\description{
This function employs the spie chart approach to combine
outcome-specific treatment hierarchies obtained in terms of the
probabilistic ranking metrics:
\itemize{
\item Surface Under the Cumulative Ranking curve (SUCRA), or
\item probability of best value (pBV).
}
}
\details{
The spie chart is a modified pie chart in which each sector corresponds to
a treatment and its radius is proportional to a ranking metric, so that
sector area encodes treatment performance (Daly et al., 2020). This function
constructs one spie chart per outcome using SUCRA or probability of best
values (pBV) metrics, where each metric determines the radius of the
corresponding sector.

An amalgamated treatment hierarchy across outcomes is then derived by
averaging the sector areas across outcomes, providing a quantitative
summary of overall treatment performance.
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Rank treatments using SUCRAs
ranks12 <- mvrank(mvnma12, method = "sucra",
  small.values = c("undes", "undes"))
ranks12

# Get amalgamated treatment hierarchy
spiechart(ranks12)

}
\references{
Daly CH, Mbuagbaw L, Thabane L, Straus SE, Hamid JS (2020):
Spie charts for quantifying treatment effectiveness and safety in multiple
outcome network meta-analysis: a proof-of-concept study.
\emph{BMC Med Res Methodol}, \bold{20}, 266
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvnma.R
\name{mvnma}
\alias{mvnma}
\alias{print.mvnma}
\title{Perform a Bayesian multivariate network meta-analysis using a
single-correlation coefficient model}
\usage{
mvnma(
  ...,
  reference.group = NULL,
  outclab = NULL,
  n.domain = NULL,
  n.chains = 4,
  n.iter = 10000,
  n.burnin = 2000,
  n.thin = max(1, floor((n.iter - n.burnin)/1000)),
  level = gs("level.ma"),
  scale.psi,
  lower.rho,
  upper.rho,
  method = "standard",
  varTE.missing = NULL,
  quiet = FALSE
)

\method{print}{mvnma}(
  x,
  digits = gs("digits"),
  digits.sd = gs("digits.sd"),
  print.sd = FALSE,
  ...
)
}
\arguments{
\item{\dots}{Additional arguments (ignored)}

\item{reference.group}{A common reference treatment across all outcomes.}

\item{outclab}{An optional argument with labels for each outcome. If NULL,
the each outcome is labelled as 'outcome_1', 'outcome_2' etc.}

\item{n.domain}{Integer indicating the position of the last outcome in the 
first outcome domain (based on the order of the supplied pairwise objects). 
Used with `method = "DM"` to restrict information sharing within outcome 
domains. Ignored when `method = "standard"`. Default is `NULL`.}

\item{n.chains}{Number of Markov chains (default=4).}

\item{n.iter}{Number of iterations (default: 10000).}

\item{n.burnin}{Number of iterations for burn-in (default: 2000).}

\item{n.thin}{Thinning rate. Default is equal to
\code{max(1, floor((n.iter - n.burnin) / 1000))}.}

\item{level}{The level used to calculate confidence intervals
for network estimates.}

\item{scale.psi}{Values for the scale parameter(s) of the Half-Normal prior
used for the heterogeneity parameters within each outcome. If NULL, all
values are set to 1. If specified, it should have a length equal to the
number of outcomes.}

\item{lower.rho}{Lower bounds for the Uniform prior(s) used for the
correlation coefficient. If NULL all bounds are set to -1.}

\item{upper.rho}{Upper bounds for the Uniform prior(s) used for the
correlation coefficient. If NULL all bounds are set to 1.}

\item{method}{A character string specifying the method to be used for model
fitting. This can be either "standard" (default), referring to the
standard bivariate model, or "DM", referring to the bivariate model based
on the DuMouchel method. The argument can be abbreviated.}

\item{varTE.missing}{Assumed (very large) variance for outcomes not reported
in a study. By default, the largest variance times 1000000 is used. This is
the same value used for argument \code{seTE.ignore} in
\code{\link[netmeta]{netimpact}} to mimicking the removal of individual
studies from the network meta-analysis.}

\item{quiet}{A logical indicating whether to print information on the
progress of the JAGS model fitting.}

\item{x}{An object of class \code{\link{mvnma}}.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.sd}{Minimal number of significant digits for standard
deviations}

\item{print.sd}{A logical specifying whether standard deviations should be
printed.}
}
\value{
The function returns an 'mvnma' object. This consists of the results for each
outcome and the correlation coefficient estimates between the combined
outcomes. The outcome-specific estimates are expressed in the format of a
list (one for each outcome) which contains:
\itemize{
\item The basic estimates (i.e. treatment vs. reference.group) for each
  outcome.
\item The heterogeneity estimates for each outcome 
\item The posterior samples corresponding to the basic estimates.
}
}
\description{
This function fits a Bayesian multivariate network meta-analysis model.
Currently, the function can simultaneously pool up to five outcomes.
Additionally, the studies to be included should be of maximum three arms.
}
\details{
The multivariate network meta-analysis (mvNMA) model supported by this
package refers to the single correlation coefficient model, interpreted as
an amalgam of within- and across-outcome correlations
(Efthimiou et al., 2015) which is a generalisation of Riley et al. (2008).

The function \code{\link{mvnma}} expects two to five outcomes /
\code{\link[meta]{pairwise}} objects. A common reference treatment across
all outcomes is required to only show comparisons with the reference in
forest plots.

The Bayesian multivariate network meta-analysis model fitted in the
\bold{mvnma} package assumes uniform priors for the between-outcome
correlation coefficients. The lower and upper bounds of these priors can be
defined using the arguments `lower.rho` and `upper.rho`. If not set, the
model will assume a `Unif (-1, 1)` prior for all correlation coefficients.
For two outcomes, a single value can be provided for `lower.rho` and
`upper.rho`. For example, `lower.rho` = 0.5 and `upper.rho` = 1 for
rho12 ~ Unif (0.5, 1)).
For more than two outcomes, the order in which the bounds are provided
matters. For example, when pooling four outcomes, the lower and
upper bounds correspond to the following order of correlation coefficients:
(rho12, rho13, rho14, rho23, rho24, rho34).

Two types of priors for the treatment effect parameters are supported via 
the argument `method`. Setting `method = "standard"` fits an mvNMA model 
using non-informative normal priors (e.g., `N(0, 10^3)`).

Alternatively, `method = "DM"` specifies the DuMouchel prior, which assumes 
constant relative treatment effects across outcomes and enables information 
sharing (DuMouchel & Harris, 1983). This may improve precision but can
introduce bias when outcomes from different domains (e.g., efficacy and
safety) are analyzed jointly.

The argument `n.domain` can be used to restrict information sharing to 
predefined outcome domains. It indicates the position (based on the order 
of the supplied pairwise objects) of the last outcome in the first domain. 
For example, with four outcomes, setting `n.domain = 2` assigns the first 
two outcomes to one domain and the remaining outcomes to a second domain. 
In this case, information is shared only within domains.

By default, `n.domain = NULL`, in which case information is shared across 
all outcomes when `method = "DM"`. This may be appropriate when all outcomes 
belong to the same domain or when cross-domain sharing is justified.

The argument `n.domain` is ignored when `method = "standard"`.
}
\examples{
# Use 'pairwise' to obtain contrast based data for the first two outcomes

# Early response
pw1 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(resp1, resp2, resp3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")

# Early remissions
pw2 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(remi1, remi2, remi3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")

# Define outcome labels
outcomes <- c("Early_Response", "Early_Remission",
  "Adverse_events", "Loss_to_follow_up", "Loss_to_follow_up_AE")

# Fit the model combining only the two efficacy outcomes
# (note, we are using only 10 iterations and 2 burnins to reduce the
#  runtime of the example; in real applications use larger numbers)
set.seed(1910)
mvnma(pw1, pw2,
  reference.group = "Placebo", outclab = outcomes[1:2],
  n.iter = 10, n.burnin = 2)

\donttest{
# Use 'pairwise' to obtain contrast based data for the third to fifth
# outcome

# Adverse events
pw3 <- pairwise(treat = list(treatment1, treatment2,treatment3),
  event = list(ae1, ae2, ae3),  n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")

# Loss to follow-up
pw4 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(loss1, loss2, loss3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")

# Loss_to_follow_up_(AE)
pw5 <- pairwise(treat = list(treatment1, treatment2, treatment3),
  event = list(loss.ae1, loss.ae2, loss.ae3), n = list(n1, n2, n3),
  studlab = id, data = Linde2015, sm = "OR")

# Fit the model combining only the two efficacy outcomes
# (note, we are using only 100 iterations and 20 burnins to reduce the
#  runtime of the example; in real applications use larger numbers)
set.seed(1909)
mvnma12 <- mvnma(pw1, pw2,
  reference.group = "Placebo", outclab = outcomes[1:2],
  n.iter = 100, n.burnin = 20)
mvnma12

# Extract treatment effect estimates and heterogeneity for Early_Response 
mvnma12$Early_Response$basic_estimates
mvnma12$Early_Response$heterogeneity

# Extract outcome correlation
mvnma12$cor

# Plot the results for efficacy outcomes
forest(mvnma12)

# Print odds ratios for efficacy outcomes
outc <- names(mvnma12)[names(mvnma12) != "cor"]
#
for (i in outc) {
  cat(paste0("\nOutcome: ", i, "\n\n"))
  print(round(exp(mvnma12[[i]]$TE.random), 2))
}

# Fit the model combining all five outcomes
# (note, we are using only 100 iterations and 20 burnins to reduce the
#  runtime of the example; in real applications use larger numbers)
set.seed(1904)
mvnma_all <- mvnma(pw1, pw2, pw3, pw4, pw5,
  reference.group = "Placebo", outclab = outcomes,
  n.iter = 100, n.burnin = 20)

# Extract treatment effect estimates and heterogeneity for Early_Response 
mvnma_all$Early_Response$basic_estimates
mvnma_all$Early_Response$heterogeneity      

# Extract outcome correlation 
mvnma_all$cor

# Plot the results for all outcomes
forest(mvnma_all)

# Print odds ratios for all outcomes
outc <- names(mvnma_all)[names(mvnma_all) != "cor"]
#
for (i in outc) {
  cat(paste0("\nOutcome: ", i, "\n\n"))
  print(round(exp(mvnma_all[[i]]$TE.random), 2))
}
}

}
\references{
DuMouchel WH, Harris JE (1983):
Bayes methods for combining the results of cancer studies in humans and
other species.
\emph{Journal of the American Statistical Association},
\bold{78}, 293--308

Efthimiou O, Mavridis D, Riley RD, Cipriani A, Salanti G (2015):
Joint synthesis of multiple correlated outcomes in networks of interventions.
\emph{Biostatistics}, 
\bold{16}, 84--97

Riley RD, Thompson JR, Abrams KR (2008):
An alternative model for bivariate random-effects meta-analysis when the
within-study correlations are unknown.
\emph{Biostatistics},
\bold{9}, 172--86
}
\seealso{
\code{\link[meta]{pairwise}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.mcmc.mvnma.R
\name{as.mcmc.mvnma}
\alias{as.mcmc.mvnma}
\alias{as.mcmc}
\title{Auxiliary function to extract the Markov Chain Monte Carlo object}
\usage{
\method{as.mcmc}{mvnma}(x, drop.reference.group = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{mvnma}.}

\item{drop.reference.group}{A logical indicating whether to drop the MCMC
samples for the reference group.}

\item{\dots}{Additional arguments passed on to
\code{\link[coda]{as.mcmc.list}}.}
}
\value{
The function returns the Markov Chain Monte Carlo object which is a list
of 'mcmc' objects.
}
\description{
Extract the Markov Chain Monte Carlo object from an R object of class
\code{mvnma}.
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

mcmc12 <- as.mcmc(mvnma12)

library(coda)
oldpar <- par(mfrow = c(3, 3))
# Traceplot
traceplot(mcmc12)
# Density plot
densplot(mcmc12)
# Traceplot and density plot together
plot(mcmc12)

# Do not print the trace plot for the reference group, i.e., placebo
mcmc12.drop <- as.mcmc(mvnma12, drop.reference.group = TRUE)
par(mfrow = c(2, 4))
# traceplot
traceplot(mcmc12.drop)
# density plot
densplot(mcmc12.drop)
# traceplot and density plot together
plot(mcmc12.drop)

par(oldpar)

}
\seealso{
\code{\link{mvnma}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.mvnma.R
\name{forest.mvnma}
\alias{forest.mvnma}
\title{Forest plot for multivariate network meta-analysis results}
\usage{
\method{forest}{mvnma}(
  x,
  backtransf = FALSE,
  separate = n_distinct(attr(x, "sm")) > 1,
  leftcols = "studlab",
  leftlabs,
  rightcols = c("effect", "ci"),
  rightlabs,
  col.study = "black",
  col.square = "black",
  col.square.lines = "black",
  col.subgroup = "black",
  squaresize = 0.7,
  header.line = TRUE,
  text.outcome = "Outcome: ",
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{mvnma}}.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{separate}{A logical indicating whether separate forest plots should be
created for outcomes.}

\item{leftcols}{A character vector specifying (additional) columns to be
printed on the left side of the forest plot or a logical value.}

\item{leftlabs}{A character vector specifying labels for (additional)
columns on left side of the forest plot.}

\item{rightcols}{A character vector specifying (additional) columns to be 
printed on the right side of the forest plot or a logical value.}

\item{rightlabs}{A character vector specifying labels for (additional)
columns on right side of the forest plot.}

\item{col.study}{The colour for treatment effect estimates and confidence
interval limits.}

\item{col.square}{The colour for squares reflecting the treatment effect
estimates.}

\item{col.square.lines}{The colour for the outer lines of squares reflecting
the treatment effect estimates.}

\item{col.subgroup}{The colour to print information on different outcomes.}

\item{squaresize}{The size of squares reflecting the treatment effect
estimates (default: 0.7).}

\item{header.line}{A logical value indicating whether to print a header
line (default: TRUE) or a character string ("both", "below", "").}

\item{text.outcome}{A text string printed before the name of the outcome.}

\item{\dots}{Additional arguments passed on to
\code{\link[meta]{forest.meta}}.}
}
\value{
A forest plot is shown in the active graphics window.
}
\description{
Draws a forest plot in the active graphics window (using grid graphics
system).
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Print multivariate network meta-analysis results
mvnma12

# Generate a forest plot with the results
forest(mvnma12)

# Generate a separate forest plot for each outcome
forest(mvnma12, separate = TRUE)

}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatplot.mvrank.R
\name{heatplot.mvrank}
\alias{heatplot.mvrank}
\title{A heatplot to visualize the ranking in a multivariate network meta-analysis}
\usage{
\method{heatplot}{mvrank}(
  x,
  sort = NULL,
  col.num = "white",
  num.size = 4.5,
  col.low = "lightblue",
  col.high = "darkblue",
  width.bar = 1.2,
  angle.x = 0,
  angle.y = 0,
  hjust.x = 0.5,
  hjust.y = 1,
  hjust.legend = 0.5,
  size.x = 8,
  size.y = 10,
  legend.position = "right",
  legend.direction = "vertical",
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{mvrank}}.}

\item{sort}{An optional argument to define an outcome to be used as a
reference when sorting the order of treatments on the x-axis.
If \code{NULL}, then the first outcome, as it appears in
\code{\link{mvrank}}, is used as the reference. The user can either
provide a single number corresponding to the position of the outcome in
the \code{\link{mvrank}} object or a character string with the name of
the outcome; can be abbreviated.}

\item{col.num}{The color of the numbers in the squares (default: "white").}

\item{num.size}{The size of the numbers in the squares (default: 4.5).}

\item{col.low}{The color for the low end of the gradient
(default: "lightblue").}

\item{col.high}{The color for the high end of the gradient
(default: "darkblue").}

\item{width.bar}{The width of the bars in the graph (default: 1.2).}

\item{angle.x}{The angle (in [0, 360]) to rotate the text appearing on
the x-axis (default: 0).}

\item{angle.y}{The angle (in [0, 360]) to rotate the text appearing on the
y-axis (default: 0).}

\item{hjust.x}{Horizontal justification for the text on the x-axis
(default: 0.5).}

\item{hjust.y}{Horizontal justification for the text on the y-axis
(default: 1).}

\item{hjust.legend}{Horizontal justification for the text in the legend
title (default: 0.5).}

\item{size.x}{Text size (in pts) on the x-axis (default: 8).}

\item{size.y}{Text size (in pts) on the y-axis (default: 10).}

\item{legend.position}{Position of the legend. Options are "top", "bottom",
"right" (default), and "left"; can be abbreviated.}

\item{legend.direction}{Direction of the legend. Options are "vertical"
(default) and "horizontal"; can be abbreviated.}

\item{\dots}{Additional arguments (ignored).}
}
\value{
A \code{ggplot} object.
}
\description{
This function produces a heatplot displaying the results of function
\code{\link{mvrank}}. The graph can be used to visualize the ranking output
when the method used to rank the treatments is either the surface under the
cumulative ranking curve (SUCRA) or the probability of best value (pBV).
}
\examples{
# Locate file "mvnma_examples.rda" with mvnma() results
.fname <- system.file("extdata/mvnma_examples.rda", package = "mvnma")
load(.fname)

# Rank the treatments
ranks_sucra <- mvrank(mvnma_all, 
  small.values = c("undes", "undes", "des", "des", "des"),
  method = "SUCRA")
ranks_sucra

# Create a heatplot sorting the results according to the first outcome
# (default), i.e., Early_Response
heatplot(ranks_sucra)

# Create a heatplot sorting the results according to the second outcome
# appearing in ranks_sucra (i.e Early_Remission)
heatplot(ranks_sucra, sort = 2)

# Create a heatplot sorting the results by explicitly mentioning the
# name of the outcome Early_Remission
heatplot(ranks_sucra, sort = "Early_Remission")

}
